@@ -1,6 +1,6 @@
 /*
  * File:	feature.h
- * $Id: feature.h,v 1.20.2.9 2000/07/05 07:26:17 gcw Exp $
+ * $Id: feature.h,v 1.37 1999/12/18 06:26:07 mason Exp $
  *
  * Compile-time configuration.
  *-----------------------------------------------------------------------
@@ -24,12 +24,6 @@
 #ifndef _FEATURE_H
 #define _FEATURE_H
 
-/*
- * Define to remove handling of extra escape sequences and other rarely
- * used features.
- */
-/* #define NO_FRILLS */
-
 /*-----------------------SCREEN OPTIONS AND COLOURS---------------------*/
 /*
  * Define the name of the environment variable to be used in
@@ -43,7 +37,7 @@
  * Avoid enabling the colour cursor (-cr, cursorColor, cursorColor2)
  */
 /* #define NO_CURSORCOLOR */
- 
+
 /*
  * Suppress use of BOLD and BLINK attributes for setting bright foreground
  * and background, respectively.  Simulate BOLD using colorBD, boldFont or
@@ -68,23 +62,17 @@
  * Also use bold font or overstrike even if we use colour for bold
  */
 #define VERYBOLD
- 
+
 /*
  * Compile without support for real bold fonts
  */
 /* #define NO_BOLDFONT */
- 
+
 /*
  * If the screen has 24 bit mode, use that even if the default is 8 bit.
  */
 #define PREFER_24BIT
 
-/*
- * Define maximum possible columns and rows
- */
-#define MAX_COLS	400
-#define MAX_ROWS	300
-
 /*
  * Define default colours for certain items.  If you have a low colour
  * display, then consider using colours which are already pre-allocated:
@@ -104,14 +92,14 @@
  *   Magenta		(#FF00FF)
  *   Cyan		(#00FFFF)
  *   White		(#FFFFFF)
- */  
+ */
 /* These colours MUST be defined */
 #define COLOR_FOREGROUND	"Black"
 #define COLOR_BACKGROUND	"White"
 #define COLOR_SCROLLBAR		"#B2B2B2"	/* scrollColor match Netscape */
 #define COLOR_SCROLLTROUGH	"#969696"
 /*
- * The cursor colours are special.  Be very careful about setting these: 
+ * The cursor colours are special.  Be very careful about setting these:
  * foreground/background colours may be modified by command line or resources
  * prior to this allocation.  Also, they are not valid if NO_CURSORCOLOR is
  * defined
@@ -135,10 +123,11 @@
  * Define where to find installed application defaults for rxvt
  * Only if USE_XGETDEFAULT is not defined.
  */
+/* #define USE_XAPPLOADDIR */
 #ifndef XAPPLOADDIR
-/* #define XAPPLOADDIR	"/usr/lib/X11/app-defaults" */
+# define XAPPLOADDIR	"/usr/lib/X11/app-defaults"
 #endif
- 
+
 /*
  * Add support for the Offix DND (Drag 'n' Drop) protocol
  */
@@ -200,12 +189,12 @@
  */
 /* #define UNSHIFTED_SCROLLKEYS */
 
-/* (Hops) Set to choose a number of lines of context between pages 
- *      (rather than a proportion (1/5) of savedlines buffer) 
+/* (Hops) Set to choose a number of lines of context between pages
+ *      (rather than a proportion (1/5) of savedlines buffer)
  *      when paging the savedlines with SHIFT-{Prior,Next} keys.
  */
 #define PAGING_CONTEXT_LINES 1 /* */
-  
+
 /*--------------------------------MOUSE---------------------------------*/
 /*
  * Disable sending escape sequences (up, down, page up/down)
@@ -246,7 +235,23 @@
 /* #define SCROLLBAR_INITIAL_DELAY 40 */
 /* #define SCROLLBAR_CONTINUOUS_DELAY 2 */
 
+/*--------------------------------MISC----------------------------------*/
+/*
+ * Disable to reset tty device to pre-incovation state
+ */
+#define RESET_TTY_TO_COMMON_DEFAULTS
+
+/*
+ * Only log in wtmp file when we're a login shell (-ls option)
+ */
+/* #define WTMP_ONLY_ON_LOGIN */
+
 /*--------------------------------BELL----------------------------------*/
+/*
+ * Disable all bell indications
+ */
+/* define NO_BELL */
+
 /*
  * Disable automatic de-iconify when a bell is received
  */
@@ -265,14 +270,14 @@
  */
 /* #define SB_WIDTH 10 */
 
-/* 
+/*
  * When using Rxvt scrollbar, clicking above or below the slider will move
  * 1/4 of the screen height, if possible.  Setting RXVT_SCROLL_FULL will move
  * it one screen height less one line, if possible
  */
 #define RXVT_SCROLL_FULL 1
 
-/* 
+/*
  * (Hops) draw an internal border line on inside edge of the scrollbar
  */
 /* #define SB_BORDER */
@@ -334,7 +339,7 @@
  */
 /* #define ENABLE_DISPLAY_ANSWER */
 
-/* 
+/*
  * Change what ESC Z transmits instead of the default "\E[?1;2c"
  */
 /* #define ESCZ_ANSWER	"\033[?1;2C" */
@@ -352,9 +357,19 @@
 #define XTERM_COLOR_CHANGE
 
 /*
- * Width of the term border
+ * Remove secondary screen's independent cursor position, a la xterm
  */
-#define BORDERWIDTH	1
+/* #define NO_SECONDARY_SCREEN_CURSOR */
+
+/*
+ * Width of the term internal border
+ */
+#define INTERNALBORDERWIDTH	2
+
+/*
+ * Width of the term external border
+ */
+#define EXTERNALBORDERWIDTH	0
 
 /*
  * Default number of lines in the scrollback buffer
@@ -375,30 +390,11 @@
  * Sizes between multi-char fonts sets (MFONT_LIST) and single-char font
  * sets (NFONT_LIST) have been matched up
  */
-#ifndef MULTICHAR_SET		/* no Kanji or Big5 or GB support */
-# define NFONTS		5
-# define FONT0_IDX	2
-# undef  MFONT_LIST
-# define NFONT_LIST	"7x14", "6x10", "6x13", "8x13", "9x15"
-#endif
-#ifdef KANJI
-# define NFONTS		5
-# define FONT0_IDX	2
-# define MFONT_LIST	"k14", "jiskan16", "jiskan18", "jiskan24", "jiskan26"
-# define NFONT_LIST	"7x14", "8x16", "9x18", "12x24", "13x26"
-#endif
-#ifdef ZH
-# define NFONTS		5
-# define FONT0_IDX	1
-# define MFONT_LIST	"taipei16", "taipeik20", "taipeik24", "taipeik20", \
-       			"taipei16"
-# define NFONT_LIST	"8x16", "10x20", "12x24", "10x20", "8x16"
-#endif
-#ifdef ZHCN		/* Here are our default GB fonts. */
-# define NFONTS		3
-# define FONT0_IDX	1
-# define MFONT_LIST	"hanzigb16st", "hanzigb24st", "hanzigb16fs"
-# define NFONT_LIST	"8x16", "12x24", "8x16"
+#define NFONTS		7
+#define FONT0_IDX	2
+#define NFONT_LIST	"7x14", "6x10", "6x13", "8x13", "8x16", "10x20", "12x24"
+#ifdef MULTICHAR_SET		/* multichar glyph language support */
+# define MFONT_LIST	NULL, NULL,  NULL, NULL, NULL, NULL, NULL
 #endif
 
 #endif
