@@ -1,7 +1,7 @@
 /*--------------------------------*-C-*---------------------------------*
  * File:	graphics.c
  *----------------------------------------------------------------------*
- * $Id: graphics.c,v 1.19 1999/04/16 05:44:06 mason Exp $
+ * $Id: graphics.c,v 1.24 1999/12/18 06:51:23 mason Exp $
  *
  * All portions of code are copyright by their respective author/s.
  * Copyright (C) 1994      Rob Nation <nation@rocket.sanders.lockheed.com>
@@ -24,7 +24,19 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *----------------------------------------------------------------------*/
 
+#include "../config.h"		/* NECESSARY */
+#ifdef RXVT_GRAPHICS
 #include "rxvt.h"		/* NECESSARY */
+
+typedef struct grwin_t {
+    Window          win;
+    int             x, y;
+    unsigned int    w, h;
+    short           screen;
+    grcmd_t        *graphics;
+    struct grwin_t *prev, *next;
+} grwin_t;
+
 #include "graphics.intpro"	/* PROTOS for internal routines */
 #include <X11/cursorfont.h>
 
@@ -42,15 +54,12 @@
 # define GRX_SCALE	10000
 #endif
 
-#ifdef RXVT_GRAPHICS
 static int      graphics_up = 0;
 static grwin_t *gr_root = NULL;
-#endif
 
 /*----------------------------------------------------------------------*
  * local functions
  */
-#ifdef RXVT_GRAPHICS
 /* ARGSUSED */
 /* INTPROTO */
 void
@@ -66,8 +75,8 @@ Gr_NewWindow(int nargs, int args[])
 	print_error("NewWindow: 4 args needed, got %d\n", nargs);
 	return;
     }
-    x = args[0] * TermWin.width / GRX_SCALE + TermWin_internalBorder;
-    y = args[1] * TermWin.height / GRX_SCALE + TermWin_internalBorder;
+    x = args[0] * TermWin.width / GRX_SCALE + TermWin.int_bwidth;
+    y = args[1] * TermWin.height / GRX_SCALE + TermWin.int_bwidth;
     w = args[2] * TermWin.width / GRX_SCALE;
     h = args[3] * TermWin.height / GRX_SCALE;
 
@@ -176,14 +185,14 @@ Gr_Geometry(grwin_t *grwin, grcmd_t *data)
 		  Xdepth);
     else			/* rxvt terminal window size */
 	tt_printf("\033G0 0 0 %d %d %d %d %ld %ld %d\n",
-		  TermWin.width - 2 * TermWin_internalBorder,
-		  TermWin.height - 2 * TermWin_internalBorder,
+		  TermWin.width - 2 * TermWin.int_bwidth,
+		  TermWin.height - 2 * TermWin.int_bwidth,
 		  TermWin.fwidth,
 		  TermWin.fheight,
 		  (long)GRX_SCALE * TermWin.fwidth
-		        / (TermWin.width - 2 * TermWin_internalBorder),
+		        / (TermWin.width - 2 * TermWin.int_bwidth),
 		  (long)GRX_SCALE * TermWin.fheight
-		        / (TermWin.height - 2 * TermWin_internalBorder),
+		        / (TermWin.height - 2 * TermWin.int_bwidth),
 		  Xdepth);
 }
 
@@ -304,7 +313,6 @@ Gr_Redraw(grwin_t *grwin)
     for (cmd = grwin->graphics; cmd != NULL; cmd = cmd->next)
 	Gr_Dispatch(grwin, cmd);
 }
-#endif				/* RXVT_GRAPHICS */
 
 /*----------------------------------------------------------------------*
  * end of static functions
@@ -314,7 +322,6 @@ Gr_Redraw(grwin_t *grwin)
 void
 Gr_ButtonReport(int but, int x, int y)
 {
-#ifdef RXVT_GRAPHICS
     grwin_t        *grwin;
 
     for (grwin = gr_root; grwin != NULL; grwin = grwin->next)
@@ -330,7 +337,6 @@ Gr_ButtonReport(int but, int x, int y)
     x = GRX_SCALE * (x - grwin->x) / grwin->w;
     y = GRX_SCALE * (y - grwin->y) / grwin->h;
     tt_printf("\033%c%ld;%d;%d;\n", but, (long)grwin->win, x, y);
-#endif
 }
 
 /* ARGSUSED */
@@ -338,7 +344,6 @@ Gr_ButtonReport(int but, int x, int y)
 void
 Gr_do_graphics(int cmd, int nargs, int args[], unsigned char *text)
 {
-#ifdef RXVT_GRAPHICS
     static Window   last_id = None;
     long            win_id;
     grwin_t        *grwin;
@@ -420,7 +425,6 @@ Gr_do_graphics(int cmd, int nargs, int args[], unsigned char *text)
 	oldcmd->next = newcmd;
     }
     Gr_Dispatch(grwin, newcmd);
-#endif
 }
 
 /* ARGSUSED */
@@ -428,7 +432,6 @@ Gr_do_graphics(int cmd, int nargs, int args[], unsigned char *text)
 void
 Gr_scroll(int count)
 {
-#ifdef RXVT_GRAPHICS
     static short    prev_start = 0;
     grwin_t        *grwin, *next;
 
@@ -447,14 +450,12 @@ Gr_scroll(int count)
 			grwin->x,
 			grwin->y + (TermWin.view_start * TermWin.fheight));
     }
-#endif
 }
 
 /* EXTPROTO */
 void
 Gr_ClearScreen(void)
 {
-#ifdef RXVT_GRAPHICS
     grwin_t        *grwin, *next;
 
     for (grwin = gr_root; grwin != NULL; grwin = next) {
@@ -467,14 +468,12 @@ Gr_ClearScreen(void)
 			      grwin->w, -grwin->y);
 	}
     }
-#endif
 }
 
 /* EXTPROTO */
 void
 Gr_ChangeScreen(void)
 {
-#ifdef RXVT_GRAPHICS
     grwin_t        *grwin, *next;
 
     for (grwin = gr_root; grwin != NULL; grwin = next) {
@@ -489,7 +488,6 @@ Gr_ChangeScreen(void)
 	    }
 	}
     }
-#endif
 }
 
 /* ARGSUSED */
@@ -497,7 +495,6 @@ Gr_ChangeScreen(void)
 void
 Gr_expose(Window win)
 {
-#ifdef RXVT_GRAPHICS
     grwin_t        *grwin;
 
     for (grwin = gr_root; grwin != NULL; grwin = grwin->next) {
@@ -506,7 +503,6 @@ Gr_expose(Window win)
 	    break;
 	}
     }
-#endif
 }
 
 /* ARGSUSED */
@@ -514,7 +510,6 @@ Gr_expose(Window win)
 void
 Gr_Resize(int w, int h)
 {
-#ifdef RXVT_GRAPHICS
     grwin_t        *grwin;
 
     for (grwin = gr_root; grwin != NULL; grwin = grwin->next) {
@@ -526,14 +521,12 @@ Gr_Resize(int w, int h)
 	}
 	Gr_Redraw(grwin);
     }
-#endif
 }
 
 /* EXTPROTO */
 void
 Gr_reset(void)
 {
-#ifdef RXVT_GRAPHICS
     grwin_t        *grwin, *next;
 
     for (grwin = gr_root; grwin != NULL; grwin = next) {
@@ -542,17 +535,13 @@ Gr_reset(void)
     }
 
     graphics_up = 0;
-#endif
 }
 
 /* EXTPROTO */
 int
 Gr_Displayed(void)
 {
-#ifdef RXVT_GRAPHICS
     return graphics_up;
-#else
-    return 0;
-#endif
 }
+#endif
 /*----------------------- end-of-file (C source) -----------------------*/
