@@ -1,7 +1,7 @@
 /*--------------------------------*-C-*---------------------------------*
  * File:	main.c
  *----------------------------------------------------------------------*
- * $Id: main.c,v 1.66.2.12 2000/10/28 05:44:28 gcw Exp $
+ * $Id: main.c,v 1.99 1999/12/18 07:41:47 mason Exp $
  *
  * All portions of code are copyright by their respective author/s.
  * Copyright (C) 1992      John Bovey, University of Canterbury
@@ -12,7 +12,7 @@
  * Copyright (C) 1997      mj olesen <olesen@me.QueensU.CA>
  *				- extensive modifications
  * Copyright (C) 1997,1998 Oezguer Kesim <kesim@math.fu-berlin.de>
- * Copyright (C) 1998      Geoff Wing <gcw@pobox.com>
+ * Copyright (C) 1998,1999 Geoff Wing <gcw@pobox.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *---------------------------------------------------------------------*/
 
+#include "../config.h"		/* NECESSARY */
 #define INTERN			/* assign all global vars to me */
 #include "rxvt.h"		/* NECESSARY */
 #include "main.intpro"		/* PROTOS for internal routines */
@@ -107,18 +108,18 @@ xerror_handler(const Display *display, const XErrorEvent *event)
 void
 color_aliases(int idx)
 {
-    if (rs[Rs_color + idx] && isdigit(*(rs[Rs_color + idx]))) {
-	int             i = atoi(rs[Rs_color + idx]);
+    if (rs.color[idx] && isdigit(*(rs.color[idx]))) {
+	int             i = atoi(rs.color[idx]);
 
 	if (i >= 8 && i <= 15) {	/* bright colors */
 	    i -= 8;
 #ifndef NO_BRIGHTCOLOR
-	    rs[Rs_color + idx] = rs[Rs_color + minBrightCOLOR + i];
+	    rs.color[idx] = rs.color[minBrightCOLOR + i];
 	    return;
 #endif
 	}
 	if (i >= 0 && i <= 7)	/* normal colors */
-	    rs[Rs_color + idx] = rs[Rs_color + minCOLOR + i];
+	    rs.color[idx] = rs.color[minCOLOR + i];
     }
 }
 
@@ -147,13 +148,13 @@ set_colorfgbg(void)
 	}
     }
 
-    p = strchr(env_colorfgbg, '=');
+    p = STRCHR(env_colorfgbg, '=');
     p++;
     if (fg >= 0)
 	sprintf(p, "%d;", fg);
     else
 	STRCPY(p, "default;");
-    p = strchr(p, '\0');
+    p = STRCHR(p, '\0');
     if (bg >= 0)
 	sprintf(p,
 #ifdef XPM_BACKGROUND
@@ -193,23 +194,23 @@ Get_Colours(void)
 	const char     *msg = "can't load color \"%s\"";
 	XColor          xcol;
 
-	if (!rs[Rs_color + i])
+	if (!rs.color[i])
 	    continue;
 
-	if (!XParseColor(Xdisplay, Xcmap, rs[Rs_color + i], &xcol)
+	if (!XParseColor(Xdisplay, Xcmap, rs.color[i], &xcol)
 	    || !XAllocColor(Xdisplay, Xcmap, &xcol)) {
-	    print_error(msg, rs[Rs_color + i]);
+	    print_error(msg, rs.color[i]);
 #ifndef XTERM_REVERSE_VIDEO
 	    if (i < 2 && (Options & Opt_reverseVideo)) {
-		rs[Rs_color + i] = def_colorName[!i];
+		rs.color[i] = def_colorName[!i];
 	    } else
 #endif
-	        rs[Rs_color + i] = def_colorName[i];
-	    if (!rs[Rs_color + i])
+	        rs.color[i] = def_colorName[i];
+	    if (!rs.color[i])
 		continue;
-	    if (!XParseColor(Xdisplay, Xcmap, rs[Rs_color + i], &xcol)
+	    if (!XParseColor(Xdisplay, Xcmap, rs.color[i], &xcol)
 		|| !XAllocColor(Xdisplay, Xcmap, &xcol)) {
-		print_error(msg, rs[Rs_color + i]);
+		print_error(msg, rs.color[i]);
 		switch (i) {
 		case Color_fg:
 		case Color_bg:
@@ -235,9 +236,9 @@ Get_Colours(void)
 	PixColors[i] = xcol.pixel;
     }
 
-    if (Xdepth <= 2 || !rs[Rs_color + Color_pointer])
+    if (Xdepth <= 2 || !rs.color[Color_pointer])
 	PixColors[Color_pointer] = PixColors[Color_fg];
-    if (Xdepth <= 2 || !rs[Rs_color + Color_border])
+    if (Xdepth <= 2 || !rs.color[Color_border])
 	PixColors[Color_border] = PixColors[Color_fg];
 
 /*
@@ -341,12 +342,12 @@ Create_Windows(int argc, const char * const *argv)
 
 #ifdef PREFER_24BIT
     attributes.background_pixel = PixColors[Color_fg];
-    attributes.border_pixel = PixColors[Color_border];
+    attributes.border_pixel = PixColors[Color_fg];
     attributes.colormap = Xcmap;
     TermWin.parent[0] = XCreateWindow(Xdisplay, Xroot,
 				   szHint.x, szHint.y,
 				   szHint.width, szHint.height,
-				   BORDERWIDTH,
+				   TermWin.ext_bwidth,
 				   Xdepth, InputOutput,
 				   Xvisual,
 				   CWBackPixel | CWBorderPixel | CWColormap,
@@ -355,14 +356,14 @@ Create_Windows(int argc, const char * const *argv)
     TermWin.parent[0] = XCreateSimpleWindow(Xdisplay, Xroot,
 					 szHint.x, szHint.y,
 					 szHint.width, szHint.height,
-					 BORDERWIDTH,
-					 PixColors[Color_border],
+					 TermWin.ext_bwidth,
+					 PixColors[Color_bg],
 					 PixColors[Color_fg]);
 #endif
-    xterm_seq(XTerm_title, rs[Rs_title]);
-    xterm_seq(XTerm_iconName, rs[Rs_iconName]);
+    xterm_seq(XTerm_title, rs.title);
+    xterm_seq(XTerm_iconName, rs.iconName);
 /* ignore warning about discarded `const' */
-    classHint.res_name = (char *) rs[Rs_name];
+    classHint.res_name = (char *) rs.name;
     classHint.res_class = (char *) APL_CLASS;
     wmHint.input = True;
     wmHint.initial_state = (Options & Opt_iconic ? IconicState : NormalState);
@@ -399,6 +400,9 @@ Create_Windows(int argc, const char * const *argv)
 				     0,
 				     PixColors[Color_fg],
 				     PixColors[Color_bg]);
+#ifdef DEBUG_X
+    XStoreName(Xdisplay, TermWin.vt, "vt window");
+#endif
 
     XDefineCursor(Xdisplay, TermWin.vt, TermWin_cursor);
     XSelectInput(Xdisplay, TermWin.vt,
@@ -412,6 +416,9 @@ Create_Windows(int argc, const char * const *argv)
 					0,
 					PixColors[Color_fg],
 					PixColors[Color_bg]);
+#ifdef DEBUG_X
+    XStoreName(Xdisplay, scrollBar.win, "scrollbar");
+#endif
 
     XDefineCursor(Xdisplay, scrollBar.win, cursor);
     XSelectInput(Xdisplay, scrollBar.win,
@@ -423,20 +430,20 @@ Create_Windows(int argc, const char * const *argv)
     }
 
 #ifdef XPM_BACKGROUND
-    if (rs[Rs_backgroundPixmap] != NULL && !(Options & Opt_transparent)) {
-	const char     *p = rs[Rs_backgroundPixmap];
+    if (rs.backgroundPixmap != NULL && !(Options & Opt_transparent)) {
+	const char     *p = rs.backgroundPixmap;
 
-	if ((p = strchr(p, ';')) != NULL) {
+	if ((p = STRCHR(p, ';')) != NULL) {
 	    p++;
 	    scale_pixmap(p);
 	}
-	set_bgPixmap(rs[Rs_backgroundPixmap]);
-	scr_touch();
+	set_bgPixmap(rs.backgroundPixmap);
+	scr_touch(True);
     }
 # ifdef XPM_BUFFERING
     else {
 	set_bgPixmap("");
-	scr_touch();
+	scr_touch(True);
     }
 # endif
 #endif
@@ -461,7 +468,9 @@ void
 resize_subwindows(int width, int height)
 {
     int             x = 0, y = 0;
+#ifdef RXVT_GRAPHICS
     int             old_width = TermWin.width, old_height = TermWin.height;
+#endif
 
     TermWin.width = TermWin.ncol * TermWin.fwidth;
     TermWin.height = TermWin.nrow * TermWin.fheight;
@@ -471,27 +480,11 @@ resize_subwindows(int width, int height)
 
 /* size and placement */
     if (scrollbar_visible()) {
-	scrollBar.beg = 0;
-	scrollBar.end = height;
-#ifndef XTERM_SCROLLBAR
-    /* arrows are as high as wide - leave 1 pixel gap */
-# ifdef NEXT_SCROLLBAR
-	scrollBar.end -= SB_BUTTON_TOTAL_HEIGHT + SB_PADDING;
-# else
-	scrollBar.beg += (SB_WIDTH + 1) + sb_shadow;
-	scrollBar.end -= (SB_WIDTH + 1) + sb_shadow;
-# endif
-#endif
+	Resize_scrollBar();
 	x = (SB_WIDTH + 2 * sb_shadow);	/* placement of vt window */
 	width -= x;
-	if ((Options & Opt_scrollBar_right) == 0)
-	    XMoveResizeWindow(Xdisplay, scrollBar.win, 0, 0,
-			      x, height);
-	else {
-	    XMoveResizeWindow(Xdisplay, scrollBar.win, width, 0,
-			      x, height);
+	if (Options & Opt_scrollBar_right)
 	    x = 0;		/* scrollbar on right so vt window at left */
-	}
     }
     { /* ONLYIF MENUBAR */
 	if (menubar_visible()) {
@@ -501,8 +494,10 @@ resize_subwindows(int width, int height)
     }
     XMoveResizeWindow(Xdisplay, TermWin.vt, x, y, width, height + 1);
 
+#ifdef RXVT_GRAPHICS
     if (old_width)
 	Gr_Resize(old_width, old_height);
+#endif
 
     scr_clear();
     resize_pixmap();
@@ -533,14 +528,6 @@ resize_window(unsigned int width, unsigned int height)
 
     new_ncol = (width - szHint.base_width) / TermWin.fwidth;
     new_nrow = (height - szHint.base_height) / TermWin.fheight;
-    if (new_ncol > MAX_COLS) {
-	new_ncol = MAX_COLS;
-	width = (MAX_COLS * TermWin.fwidth) + szHint.base_width;
-    }
-    if (new_nrow > MAX_ROWS) {
-	new_nrow = MAX_ROWS;
-	height = (MAX_ROWS * TermWin.fheight) + szHint.base_height;
-    }
     if (old_height == -1
 	|| (new_ncol != TermWin.ncol)
 	|| (new_nrow != TermWin.nrow)) {
@@ -575,8 +562,6 @@ set_widthheight(unsigned int width, unsigned int height)
 {
     XWindowAttributes wattr;
 
-    MIN_IT(width, (MAX_COLS * TermWin.fwidth));
-    MIN_IT(height, (MAX_ROWS * TermWin.fheight));
     if (width == 0 || height == 0) {
 	XGetWindowAttributes(Xdisplay, Xroot, &wattr);
 	if (width == 0)
@@ -609,15 +594,15 @@ szhints_set(void)
     szHint.win_gravity = NorthWestGravity;
     szHint.min_aspect.x = szHint.min_aspect.y = 1;
 
-    flags = (rs[Rs_geometry] ?
-	     XParseGeometry(rs[Rs_geometry], &x, &y, &width, &height) : 0);
+    flags = (rs.geometry ? XParseGeometry(rs.geometry, &x, &y, &width, &height)
+		         : 0);
 
     if (flags & WidthValue) {
-	TermWin.ncol = min(width, MAX_COLS);
+	TermWin.ncol = width;
 	szHint.flags |= USSize;
     }
     if (flags & HeightValue) {
-	TermWin.nrow = min(height, MAX_ROWS);
+	TermWin.nrow = height;
 	szHint.flags |= USSize;
     }
     TermWin.width = TermWin.ncol * TermWin.fwidth;
@@ -626,8 +611,8 @@ szhints_set(void)
 
     if (flags & XValue) {
 	if (flags & XNegative) {
-	    x += (DisplayWidth(Xdisplay, Xscreen)
-		  - (szHint.width + TermWin_internalBorder));
+	    x += (DisplayWidth(Xdisplay, Xscreen) - szHint.width
+		  - 2 * TermWin.ext_bwidth);
 	    szHint.win_gravity = NorthEastGravity;
 	}
 	szHint.x = x;
@@ -635,10 +620,12 @@ szhints_set(void)
     }
     if (flags & YValue) {
 	if (flags & YNegative) {
-	    y += (DisplayHeight(Xdisplay, Xscreen)
-		  - (szHint.height + TermWin_internalBorder));
-	    szHint.win_gravity = (szHint.win_gravity == NorthEastGravity ?
-				  SouthEastGravity : SouthWestGravity);
+	    y += (DisplayHeight(Xdisplay, Xscreen) - szHint.height
+		  - 2 * TermWin.ext_bwidth);
+	    if (szHint.win_gravity == NorthEastGravity)
+		szHint.win_gravity = SouthEastGravity;
+	    else
+		szHint.win_gravity = SouthWestGravity;
 	}
 	szHint.y = y;
 	szHint.flags |= USPosition;
@@ -649,8 +636,8 @@ szhints_set(void)
 void
 szhints_recalc(void)
 {
-    szHint.base_width = (2 * TermWin_internalBorder);
-    szHint.base_height = (2 * TermWin_internalBorder);
+    szHint.base_width = 2 * TermWin.int_bwidth;
+    szHint.base_height = 2 * TermWin.int_bwidth;
     szHint.base_width += (scrollbar_visible() ? (SB_WIDTH + 2 * sb_shadow) : 0);
     szHint.base_height += (menubar_visible() ? menuBar_TotalHeight() : 0);
     szHint.width_inc = TermWin.fwidth;
@@ -671,9 +658,9 @@ set_title(const char *str)
 #else
     char           *name;
 
-    if (XFetchName(Xdisplay, TermWin.parent[0], &name))
+    if (XFetchName(Xdisplay, TermWin.parent[0], &name) == 0)
 	name = NULL;
-    if (name == NULL || strcmp(name, str))
+    if (name == NULL || STRCMP(name, str))
 	XStoreName(Xdisplay, TermWin.parent[0], str);
     if (name)
 	XFree(name);
@@ -691,7 +678,7 @@ set_iconName(const char *str)
 
     if (XGetIconName(Xdisplay, TermWin.parent[0], &name))
 	name = NULL;
-    if (name == NULL || strcmp(name, str))
+    if (name == NULL || STRCMP(name, str))
 	XSetIconName(Xdisplay, TermWin.parent[0], str);
     if (name)
 	XFree(name);
@@ -783,16 +770,18 @@ set_window_color(int idx, const char *color)
  *      50 = change font
  *
  * rxvt extensions:
- *      10 = menu
+ *      10 = menu (may change in future)
  *      20 = bg pixmap
  *      39 = change default fg color
  *      49 = change default bg color
+ *      55 = dump scrollback buffer and all of screen
  */
 /* EXTPROTO */
 void
 xterm_seq(int op, const char *str)
 {
     int             changed = 0;
+    int             fd;
 
     assert(str != NULL);
     switch (op) {
@@ -815,15 +804,15 @@ xterm_seq(int op, const char *str)
 	if (*str != ';') {
 	    scale_pixmap("");	/* reset to default scaling */
 	    set_bgPixmap(str);	/* change pixmap */
-	    scr_touch();
+	    scr_touch(True);
 	}
-	while ((str = strchr(str, ';')) != NULL) {
+	while ((str = STRCHR(str, ';')) != NULL) {
 	    str++;
 	    changed += scale_pixmap(str);
 	}
 	if (changed) {
 	    resize_pixmap();
-	    scr_touch();
+	    scr_touch(True);
 	}
 	break;
 
@@ -838,6 +827,12 @@ xterm_seq(int op, const char *str)
     case XTerm_font:
 	change_font(0, str);
 	break;
+    case XTerm_dumpscreen:	/* no error notices */
+	if ((fd = open(str, O_RDWR | O_CREAT | O_EXCL, 0600)) >= 0) {
+	    scr_dump(fd);
+	    close(fd);
+	}
+	break;
     }
 }
 
@@ -853,14 +848,19 @@ void
 change_font(int init, const char *fontname)
 {
     const char     *msg = "can't load font \"%s\"";
-    int             idx = 0;	/* index into rs[Rs_font] */
-    int             recheckfonts;
+    int             fh, fw, recheckfonts;
+    int             idx = 0;	/* index into rs.font[] */
     XFontStruct    *xfont;
     static char    *newfont[NFONTS];
     static int      fnum;		/* logical font number */
 #ifndef NO_BOLDFONT
     static XFontStruct *boldFont;
 #endif
+#ifdef MULTICHAR_SET
+    int             i;
+    char           *c, *enc;
+    char            tmpbuf[64];
+#endif
 
 #if (FONT0_IDX == 0)
 # define IDX2FNUM(i)	(i)
@@ -912,7 +912,7 @@ change_font(int init, const char *fontname)
 	    if (fontname != NULL) {
 	    /* search for existing fontname */
 		for (idx = 0; idx < NFONTS; idx++) {
-		    if (!strcmp(rs[Rs_font + idx], fontname)) {
+		    if (!STRCMP(rs.font[idx], fontname)) {
 			fnum = IDX2FNUM(idx);
 			fontname = NULL;
 			break;
@@ -932,7 +932,7 @@ change_font(int init, const char *fontname)
 	    if (!xfont)
 		return;
 
-	    name = MALLOC(strlen(fontname + 1) * sizeof(char));
+	    name = MALLOC(STRLEN(fontname + 1) * sizeof(char));
 
 	    if (name == NULL) {
 		XFreeFont(Xdisplay, xfont);
@@ -942,20 +942,20 @@ change_font(int init, const char *fontname)
 	    if (newfont[idx] != NULL)
 		FREE(newfont[idx]);
 	    newfont[idx] = name;
-	    rs[Rs_font + idx] = newfont[idx];
+	    rs.font[idx] = newfont[idx];
 	}
     }
     if (TermWin.font)
 	XFreeFont(Xdisplay, TermWin.font);
 
 /* load font or substitute */
-    xfont = XLoadQueryFont(Xdisplay, rs[Rs_font + idx]);
+    xfont = XLoadQueryFont(Xdisplay, rs.font[idx]);
     if (!xfont) {
-	print_error(msg, rs[Rs_font + idx]);
-	rs[Rs_font + idx] = "fixed";
-	xfont = XLoadQueryFont(Xdisplay, rs[Rs_font + idx]);
+	print_error(msg, rs.font[idx]);
+	rs.font[idx] = "fixed";
+	xfont = XLoadQueryFont(Xdisplay, rs.font[idx]);
 	if (!xfont) {
-	    print_error(msg, rs[Rs_font + idx]);
+	    print_error(msg, rs.font[idx]);
 	    goto Abort;
 	}
     }
@@ -963,62 +963,32 @@ change_font(int init, const char *fontname)
 
 #ifndef NO_BOLDFONT
 /* fail silently */
-    if (init && rs[Rs_boldFont] != NULL)
-	boldFont = XLoadQueryFont(Xdisplay, rs[Rs_boldFont]);
+    if (init && rs.boldFont != NULL)
+	boldFont = XLoadQueryFont(Xdisplay, rs.boldFont);
 #endif
 
-#ifdef MULTICHAR_SET
-    if (TermWin.mfont)
-	XFreeFont(Xdisplay, TermWin.mfont);
-
-/* load font or substitute */
-    xfont = XLoadQueryFont(Xdisplay, rs[Rs_mfont + idx]);
-    if (!xfont) {
-	print_error(msg, rs[Rs_mfont + idx]);
-#ifdef ZHCN
-	rs[Rs_mfont + idx] = "-*-16-*-gb2312*-*";
-#else
-	rs[Rs_mfont + idx] = "k14";
-#endif
-	xfont = XLoadQueryFont(Xdisplay, rs[Rs_mfont + idx]);
-	if (!xfont) {
-	    print_error(msg, rs[Rs_mfont + idx]);
-	    goto Abort;
-	}
-    }
-    TermWin.mfont = xfont;
-#endif				/* MULTICHAR_SET */
-
 /* alter existing GC */
     if (!init) {
 	XSetFont(Xdisplay, TermWin.gc, TermWin.font->fid);
 	menubar_expose();
     }
+
 /* set the sizes */
-    {
-	int             i, cw, fh, fw = 0;
-
-	fw = get_fontwidest(TermWin.font);
-	fh = TermWin.font->ascent + TermWin.font->descent;
-
-	if (fw == TermWin.font->min_bounds.width)
-	    TermWin.fprop = 0;	/* Mono-spaced (fixed width) font */
-	else
-	    TermWin.fprop = 1;	/* Proportional font */
-
-	recheckfonts = !(fw == TermWin.fwidth && fh == TermWin.fheight);
-
-	TermWin.fwidth = fw;
-	TermWin.fheight = fh;
-    }
+    fw = get_fontwidest(TermWin.font);
+    fh = TermWin.font->ascent + TermWin.font->descent;
+    if (fw == TermWin.font->min_bounds.width)
+	TermWin.fprop = 0;	/* Mono-spaced (fixed width) font */
+    else
+	TermWin.fprop = 1;	/* Proportional font */
+    recheckfonts = !(fw == TermWin.fwidth && fh == TermWin.fheight);
+    TermWin.fwidth = fw;
+    TermWin.fheight = fh;
 
 /* check that size of boldFont is okay */
 #ifndef NO_BOLDFONT
     if (recheckfonts) {
 	TermWin.boldFont = NULL;
 	if (boldFont != NULL) {
-	    int             fh, fw;
-
 	    fw = get_fontwidest(boldFont);
 	    fh = boldFont->ascent + boldFont->descent;
 	    if (fw <= TermWin.fwidth && fh <= TermWin.fheight)
@@ -1029,18 +999,59 @@ change_font(int init, const char *fontname)
 #endif				/* NO_BOLDFONT */
 
 #ifdef MULTICHAR_SET
-    if (recheckfonts)
-    /* TODO: XXX: This could be much better? */
-	if (TermWin.mfont != NULL) {
-	    int             fh, fw;
+    if (TermWin.mfont)
+	XFreeFont(Xdisplay, TermWin.mfont);
 
+/* load font or substitute */
+    if (rs.mfont[idx] == NULL
+	|| (xfont = XLoadQueryFont(Xdisplay, rs.mfont[idx])) == NULL) {
+    /* TODO: improve this */
+	switch(encoding_method) {
+	case GB:
+	    c = "-*-%.2d-*-gb2312*-*";
+	    enc = "GB";
+	    break;
+	case BIG5:
+	    c = "-*-%.2d-*-big5*-*";
+	    enc = "BIG5";
+	    break;
+	case EUCJ:
+	case SJIS:
+	    c = "-*-%.2d-*-jisx0208*-*";
+	    enc = "EUCJ/SJIS";
+	    break;
+	case EUCKR:
+	    c = "-*-%.2d-*-ksc5601*-*";
+	    enc = "EUCKR";
+	/* FALLTHROUGH */
+	default:
+	    break;
+	}
+	for (i = 0; i < fh / 2; i++) {
+	    sprintf(tmpbuf, c, fh - i);
+	    xfont = XLoadQueryFont(Xdisplay, tmpbuf);
+	    if (xfont) {
+		rs.mfont[idx] = MALLOC(STRLEN(tmpbuf) + 1);
+		STRCPY(rs.mfont[idx], tmpbuf);
+		break;
+	    }
+	}
+	if (xfont == NULL)
+	    print_error("no similar multichar font: encoding %s; size %d",
+			enc, fh);
+    }
+    TermWin.mfont = xfont;
+
+    if (recheckfonts)
+    /* XXX: This checks what? */
+	if (TermWin.mfont != NULL) {
 	    fw = get_fontwidest(TermWin.mfont);
 	    fh = TermWin.mfont->ascent + TermWin.mfont->descent;
-	    if (fw <= TermWin.fwidth && fh <= TermWin.fheight)
-		/* WHAT TO DO!! */ ;
+	    if (fw > (TermWin.fwidth * 2) || fh > TermWin.fheight)
+		TermWin.mfont = NULL;
 	    TermWin.mprop = !(fw == TermWin.fwidth /* && fh == TermWin.fheight */ );
 	}
-#endif
+#endif				/* MULTICHAR_SET */
 
     set_colorfgbg();
 
@@ -1049,7 +1060,7 @@ change_font(int init, const char *fontname)
 
     if (!init) {
 	resize_all_windows();
-	scr_touch();
+	scr_touch(True);
     }
     return;
   Abort:
@@ -1060,6 +1071,7 @@ change_font(int init, const char *fontname)
 #undef FNUM_RANGE
     /* NOTREACHED */
 }
+#ifdef STRICT_FONT_CHECKING
 /* INTPROTO */
 int
 get_fontwidest(XFontStruct *f)
@@ -1069,23 +1081,28 @@ get_fontwidest(XFontStruct *f)
     if (f->min_bounds.width == f->max_bounds.width)
 	return f->min_bounds.width;
     if (f->per_char == NULL)
-	return 0;
+	return f->max_bounds.width;
     for (i = f->max_char_or_byte2 - f->min_char_or_byte2; --i >= 0; ) {
 	cw = f->per_char[i].width;
 	MAX_IT(fw, cw);
     }
     return fw;
 }
+#endif
 /* ------------------------------------------------------------------------- */
 /* INTPROTO */
 void
 init_vars(void)
 {
+    Xdisplay = NULL;
     Options = Opt_scrollBar | Opt_scrollTtyOutput;
     sb_shadow = 0;
     TermWin.ncol = 80;
     TermWin.nrow = 24;
     TermWin.mapped = 0;
+    TermWin.int_bwidth = INTERNALBORDERWIDTH;
+    TermWin.ext_bwidth = EXTERNALBORDERWIDTH;
+    TermWin.saveLines = SAVELINES;
     want_refresh = 1;
     scrollBar.win = 0;
 #if (MENUBAR_MAX)
@@ -1115,13 +1132,13 @@ init_resources(int argc, const char * const *argv)
     int             i, r_argc;
     char           *val;
     const char     *tmp;
-    const char    **cmd_argv, **r_argv;
+    const char    **cmd_argv, **r_argv, **t2;
 
 /*
  * Look for -exec option.  Find => split and make cmd_argv[] of command args
  */
     for (r_argc = 0; r_argc < argc; r_argc++)
-	if (!strcmp(argv[r_argc], "-e") || !strcmp(argv[r_argc], "-exec"))
+	if (!STRCMP(argv[r_argc], "-e") || !STRCMP(argv[r_argc], "-exec"))
 	    break;
     r_argv = (const char **) MALLOC(sizeof(char *) * (r_argc + 1));
     for (i = 0; i < r_argc; i++)
@@ -1137,115 +1154,129 @@ init_resources(int argc, const char * const *argv)
     }
 
 /* clear all resources */
-    for (i = 0; i < TOTAL_RS; i++)
-	rs[i] = NULL;
+    for (i = 0, t2 = (const char **)&rs; i < sizeof(rs) / sizeof(char *); i++)
+	t2[i] = NULL;
 
-    rs[Rs_name] = my_basename(argv[0]);
+    rs.name = r_basename(argv[0]);
+    if (cmd_argv != NULL && cmd_argv[0] != NULL)
+	rs.iconName = rs.title = r_basename(cmd_argv[0]);
 /*
  * Open display, get options/resources and create the window
  */
-    if ((rs[Rs_display_name] = getenv("DISPLAY")) == NULL)
-	rs[Rs_display_name] = ":0";
-#ifdef LOCAL_X_IS_UNIX
-    if (strncmp(rs[Rs_display_name], ":0", 2) == 0)
-	rs[Rs_display_name] = "unix:0";
-#endif
+    if ((rs.display_name = getenv("DISPLAY")) == NULL)
+	rs.display_name = ":0";
 
     get_options(r_argc, r_argv);
 
     FREE(r_argv);
 
-    if ((Xdisplay = XOpenDisplay(rs[Rs_display_name])) == NULL) {
-	print_error("can't open display %s", rs[Rs_display_name]);
+#ifdef LOCAL_X_IS_UNIX
+    if (rs.display_name[0] == ':') {
+	val = MALLOC(5 + STRLEN(rs.display_name));
+	STRCPY(val, "unix");
+	STRCAT(val, rs.display_name);
+	Xdisplay = XOpenDisplay(val);
+	FREE(val);
+    }
+#endif
+	
+    if (Xdisplay == NULL
+	&& (Xdisplay = XOpenDisplay(rs.display_name)) == NULL) {
+	print_error("can't open display %s", rs.display_name);
 	exit(EXIT_FAILURE);
     }
 #ifdef INEXPENSIVE_LOCAL_X_CALLS
     /* it's hard to determine further if we're on a local display or not */
-    display_is_local = rs[Rs_display_name][0] == ':' ? 1 : 0;
+    if (rs.display_name[0] == ':'
+	|| STRNCMP(rs.display_name, "unix:", 5))
+	display_is_local = 1;
+    else
+	display_is_local = 0;
 #endif
 
-    extract_resources(Xdisplay, rs[Rs_name]);
+    extract_resources(Xdisplay, rs.name);
 
 #if ! defined(XTERM_SCROLLBAR) && ! defined(NEXT_SCROLLBAR)
     if (!(Options & Opt_scrollBar_floating))
 	sb_shadow = SHADOW;
 #endif
-    
+
 /*
  * set any defaults not already set
  */
-    if (cmd_argv && cmd_argv[0]) {
-	if (!rs[Rs_title])
-	    rs[Rs_title] = my_basename(cmd_argv[0]);
-	if (!rs[Rs_iconName])
-	    rs[Rs_iconName] = rs[Rs_title];
-    } else {
-	if (!rs[Rs_title])
-	    rs[Rs_title] = rs[Rs_name];
-	if (!rs[Rs_iconName])
-	    rs[Rs_iconName] = rs[Rs_name];
-    }
-    if (!rs[Rs_saveLines] || (TermWin.saveLines = atoi(rs[Rs_saveLines])) < 0)
-	TermWin.saveLines = SAVELINES;
+    if (!rs.title)
+	rs.title = rs.name;
+    if (!rs.iconName)
+	rs.iconName = rs.title;
+    if (rs.saveLines && (i = atoi(rs.saveLines)) >= 0)
+	TermWin.saveLines = (int16_t) i;
+#ifndef NO_FRILLS
+    if (rs.int_bwidth && (i = atoi(rs.int_bwidth)) >= 0)
+	TermWin.int_bwidth = (int16_t) i;
+    if (rs.ext_bwidth && (i = atoi(rs.ext_bwidth)) >= 0)
+	TermWin.ext_bwidth = (int16_t) i;
+#endif
 
 /* no point having a scrollbar without having any scrollback! */
     if (!TermWin.saveLines)
 	Options &= ~Opt_scrollBar;
 
 #ifdef PRINTPIPE
-    if (!rs[Rs_print_pipe])
-	rs[Rs_print_pipe] = PRINTPIPE;
+    if (!rs.print_pipe)
+	rs.print_pipe = PRINTPIPE;
 #endif
-    if (!rs[Rs_cutchars])
-	rs[Rs_cutchars] = CUTCHARS;
+    if (!rs.cutchars)
+	rs.cutchars = CUTCHARS;
 #ifndef NO_BACKSPACE_KEY
-    if (!rs[Rs_backspace_key])
+    if (!rs.backspace_key)
 # ifdef DEFAULT_BACKSPACE
 	key_backspace = DEFAULT_BACKSPACE;
 # else
 	key_backspace = "DEC";	/* can toggle between \033 or \177 */
 # endif
     else {
-	val = strdup(rs[Rs_backspace_key]);
+	val = strdup(rs.backspace_key);
+	(void) Str_trim(val);
 	(void) Str_escaped(val);
 	key_backspace = val;
     }
 #endif
 #ifndef NO_DELETE_KEY
-    if (!rs[Rs_delete_key])
+    if (!rs.delete_key)
 # ifdef DEFAULT_DELETE
 	key_delete = DEFAULT_DELETE;
 # else
 	key_delete = "\033[3~";
 # endif
     else {
-	val = strdup(rs[Rs_delete_key]);
+	val = strdup(rs.delete_key);
+	(void) Str_trim(val);
 	(void) Str_escaped(val);
 	key_delete = val;
     }
 #endif
 
-    if (rs[Rs_selectstyle]) {
-	if (strncasecmp(rs[Rs_selectstyle], "oldword", 7) == 0)
+    if (rs.selectstyle) {
+	if (STRNCASECMP(rs.selectstyle, "oldword", 7) == 0)
 	    selection_style = OLD_WORD_SELECT;
 #ifndef NO_OLD_SELECTION
-	else if (strncasecmp(rs[Rs_selectstyle], "old", 3) == 0)
+	else if (STRNCASECMP(rs.selectstyle, "old", 3) == 0)
 	    selection_style = OLD_SELECT;
 #endif
     }
 
 #ifndef NO_BOLDFONT
-    if (rs[Rs_font] == NULL && rs[Rs_boldFont] != NULL) {
-	rs[Rs_font] = rs[Rs_boldFont];
-	rs[Rs_boldFont] = NULL;
+    if (rs.font[0] == NULL && rs.boldFont != NULL) {
+	rs.font[0] = rs.boldFont;
+	rs.boldFont = NULL;
     }
 #endif
     for (i = 0; i < NFONTS; i++) {
-	if (!rs[Rs_font + i])
-	    rs[Rs_font + i] = def_fontName[i];
+	if (!rs.font[i])
+	    rs.font[i] = def_fontName[i];
 #ifdef MULTICHAR_SET
-	if (!rs[Rs_mfont + i])
-	    rs[Rs_mfont + i] = def_mfontName[i];
+	if (!rs.mfont[i])
+	    rs.mfont[i] = def_mfontName[i];
 #endif
     }
     TermWin.fontset = NULL;
@@ -1253,21 +1284,21 @@ init_resources(int argc, const char * const *argv)
 #ifdef XTERM_REVERSE_VIDEO
 /* this is how xterm implements reverseVideo */
     if (Options & Opt_reverseVideo) {
-	if (!rs[Rs_color + Color_fg])
-	    rs[Rs_color + Color_fg] = def_colorName[Color_bg];
-	if (!rs[Rs_color + Color_bg])
-	    rs[Rs_color + Color_bg] = def_colorName[Color_fg];
+	if (!rs.color[Color_fg])
+	    rs.color[Color_fg] = def_colorName[Color_bg];
+	if (!rs.color[Color_bg])
+	    rs.color[Color_bg] = def_colorName[Color_fg];
     }
 #endif
 
     for (i = 0; i < NRS_COLORS; i++)
-	if (!rs[Rs_color + i])
-	    rs[Rs_color + i] = def_colorName[i];
+	if (!rs.color[i])
+	    rs.color[i] = def_colorName[i];
 
 #ifndef XTERM_REVERSE_VIDEO
 /* this is how we implement reverseVideo */
     if (Options & Opt_reverseVideo)
-	SWAP_IT(rs[Rs_color + Color_fg], rs[Rs_color + Color_bg], tmp);
+	SWAP_IT(rs.color[Color_fg], rs.color[Color_bg], tmp);
 #endif
 
 /* convenient aliases for setting fg/bg to colors */
@@ -1309,14 +1340,14 @@ init_env(void)
  *
  * Giving out the display_name also affords a potential security hole
  */
-    rs[Rs_display_name] = (const char *) val = network_display(rs[Rs_display_name]);
+    rs.display_name = (const char *) val = network_display(rs.display_name);
     if (val == NULL)
 #endif				/* DISPLAY_IS_IP */
 	val = XDisplayString(Xdisplay);
-    if (rs[Rs_display_name] == NULL)
-	rs[Rs_display_name] = val;	/* use broken `:0' value */
+    if (rs.display_name == NULL)
+	rs.display_name = val;	/* use broken `:0' value */
 
-    i = strlen(val);
+    i = STRLEN(val);
     env_display = MALLOC((i + 9) * sizeof(char));
     sprintf(env_display, "DISPLAY=%s", val);
 
@@ -1343,13 +1374,19 @@ init_env(void)
 	putenv("COLORTERM=" COLORTERMENV "-mono");
     else
 	putenv("COLORTERM=" COLORTERMENVFULL);
-    if (rs[Rs_term_name] != NULL) {
-	env_term = MALLOC((strlen(rs[Rs_term_name]) + 6) * sizeof(char));
-	sprintf(env_term, "TERM=%s", rs[Rs_term_name]);
+    if (rs.term_name != NULL) {
+	env_term = MALLOC((STRLEN(rs.term_name) + 6) * sizeof(char));
+	sprintf(env_term, "TERM=%s", rs.term_name);
 	putenv(env_term);
-    } else {
+    } else
 	putenv("TERM=" TERMENV);
-    }
+
+#ifdef HAVE_UNSETENV
+/* avoid passing old settings and confusing term size */
+    unsetenv("LINES");
+    unsetenv("COLUMNS");
+    unsetenv("TERMCAP");	/* terminfo should be okay */
+#endif				/* HAVE_UNSETENV */
 }
 
 /* ------------------------------------------------------------------------- */
@@ -1374,7 +1411,7 @@ main(int argc, const char * const *argv)
     cmd_argv = init_resources(argc, argv);
 
 #if (MENUBAR_MAX)
-    menubar_read(rs[Rs_menu]);
+    menubar_read(rs.menu);
 #endif
     scrollbar_mapping(Options & Opt_scrollBar);
 
@@ -1392,18 +1429,17 @@ main(int argc, const char * const *argv)
     XSetErrorHandler((XErrorHandler) xerror_handler);
 #endif
 
-    if (scrollbar_visible())
+    if (scrollbar_visible()) {
+	Resize_scrollBar();
 	XMapWindow(Xdisplay, scrollBar.win);
+    }
 #if (MENUBAR_MAX)
     if (menubar_visible())
 	XMapWindow(Xdisplay, menuBar.win);
 #endif
 #ifdef TRANSPARENT
-    if (Options & Opt_transparent) {
-	XSetWindowBackgroundPixmap(Xdisplay, TermWin.parent[0], ParentRelative);
-	XSetWindowBackgroundPixmap(Xdisplay, TermWin.vt, ParentRelative);
-	XSelectInput(Xdisplay, Xroot, PropertyChangeMask);
-    }
+    if (Options & Opt_transparent)
+	check_our_parents(True);
 #endif
     XMapWindow(Xdisplay, TermWin.vt);
     XMapWindow(Xdisplay, TermWin.parent[0]);
