@@ -1,7 +1,7 @@
 /*--------------------------------*-C-*---------------------------------*
  * File:	menubar.c
  *----------------------------------------------------------------------*
- * $Id: menubar.c,v 1.27 1999/01/23 14:26:36 mason Exp $
+ * $Id: menubar.c,v 1.32 1999/10/03 12:01:56 mason Exp $
  *
  * Copyright (C) 1997,1998  mj olesen <olesen@me.QueensU.CA>
  *
@@ -23,11 +23,12 @@
  * summary that appears at the end of this file was taken from there.
  *----------------------------------------------------------------------*/
 
+#include "../config.h"		/* NECESSARY */
 #include "rxvt.h"		/* NECESSARY */
+#ifdef MENUBAR
 #include "version.h"
 #include "menubar.h"
 
-#ifdef MENUBAR
 #define Menu_PixelWidth(menu)					\
     (2 * SHADOW + Width2Pixel ((menu)->width + 3 * HSPACE))
 
@@ -37,7 +38,7 @@ static int      menu_readonly = 1;	/* okay to alter menu? */
 static int      Arrows_x = 0;
 static const struct {
     char            name;	/* (l)eft, (u)p, (d)own, (r)ight */
-    unsigned char   str[4];	/* str[0] = strlen (str+1) */
+    unsigned char   str[5];	/* str[0] = STRLEN (str+1) */
 } Arrows[NARROWS] = {
     { 'l', "\003\033[D" },
     { 'u', "\003\033[A" },
@@ -54,11 +55,9 @@ static bar_t   *CurrentBar = &BarList;
 #endif				/* (MENUBAR_MAX > 1) */
 
 static menu_t  *ActiveMenu = NULL;	/* currently active menu */
-#endif
 
 /*}}} */
 
-#ifdef MENUBAR
 /*
  * find an item called NAME in MENU
  */
@@ -74,17 +73,15 @@ menuitem_find(const menu_t *menu, const char *name)
 /* find the last item in the menu, this is good for separators */
     for (item = menu->tail; item != NULL; item = item->prev) {
 	if (item->entry.type == MenuSubMenu) {
-	    if (!strcmp(name, (item->entry.submenu.menu)->name))
+	    if (!STRCMP(name, (item->entry.submenu.menu)->name))
 		break;
 	} else if ((isSeparator(name) && isSeparator(item->name))
-		   || !strcmp(name, item->name))
+		   || !STRCMP(name, item->name))
 	    break;
     }
     return item;
 }
-#endif
 
-#ifdef MENUBAR
 /*
  * unlink ITEM from its MENU and free its memory
  */
@@ -125,9 +122,7 @@ menuitem_free(menu_t *menu, menuitem_t *item)
 	FREE(item->name2);
     FREE(item);
 }
-#endif
 
-#ifdef MENUBAR
 /*
  * sort command vs. terminal actions and
  * remove the first character of STR if it's '\0'
@@ -139,7 +134,7 @@ action_type(action_t *action, unsigned char *str)
     unsigned int    len;
 
 #if defined (DEBUG_MENU) || defined (DEBUG_MENUARROWS)
-    len = strlen(str);
+    len = STRLEN(str);
     fprintf(stderr, "(len %d) = %s\n", len, str);
 #else
     len = Str_escaped((char *)str);
@@ -168,9 +163,7 @@ action_type(action_t *action, unsigned char *str)
 
     return 0;
 }
-#endif
 
-#ifdef MENUBAR
 /* INTPROTO */
 int
 action_dispatch(action_t *action)
@@ -190,9 +183,7 @@ action_dispatch(action_t *action)
     }
     return 0;
 }
-#endif
 
-#ifdef MENUBAR
 /* return the arrow index corresponding to NAME */
 /* INTPROTO */
 int
@@ -205,9 +196,7 @@ menuarrow_find(char name)
 	    return i;
     return -1;
 }
-#endif
 
-#ifdef MENUBAR
 /* free the memory associated with arrow NAME of the current menubar */
 /* INTPROTO */
 void
@@ -235,9 +224,7 @@ menuarrow_free(char name)
 	    menuarrow_free(Arrows[i].name);
     }
 }
-#endif
 
-#ifdef MENUBAR
 /* INTPROTO */
 void
 menuarrow_add(char *string)
@@ -283,19 +270,19 @@ menuarrow_add(char *string)
 	cur->len = 0;
 
 	if (cur == &end) {
-	    p = strchr(string, '\0');
+	    p = STRCHR(string, '\0');
 	} else {
 	    char           *next = string;
 
 	    while (1) {
-		p = strchr(next, '<');
+		p = STRCHR(next, '<');
 		if (p != NULL) {
 		    if (p[1] && p[2] == '>')
 			break;
 		/* parsed */
 		} else {
 		    if (beg.str == NULL)	/* no end needed */
-			p = strchr(next, '\0');
+			p = STRCHR(next, '\0');
 		    break;
 		}
 		next = (p + 1);
@@ -362,9 +349,7 @@ menuarrow_add(char *string)
 	    FREE(str);
     }
 }
-#endif
 
-#ifdef MENUBAR
 /* INTPROTO */
 menuitem_t     *
 menuitem_add(menu_t *menu, const char *name, const char *name2, const char *action)
@@ -410,7 +395,7 @@ menuitem_add(menu_t *menu, const char *name, const char *name2, const char *acti
     item->len2 = 0;
     item->name2 = NULL;
 
-    len = strlen(name);
+    len = STRLEN(name);
     item->name = MALLOC(len + 1);
     if (item->name != NULL) {
 	STRCPY(item->name, name);
@@ -438,7 +423,7 @@ menuitem_add(menu_t *menu, const char *name, const char *name2, const char *acti
  */
   Item_Found:
     if (name2 != NULL && item->name2 == NULL) {
-	len = strlen(name2);
+	len = STRLEN(name2);
 	if (len == 0 || (item->name2 = MALLOC(len + 1)) == NULL) {
 	    len = 0;
 	    item->name2 = NULL;
@@ -448,7 +433,7 @@ menuitem_add(menu_t *menu, const char *name, const char *name2, const char *acti
 	item->len2 = len;
     }
     item->entry.type = MenuLabel;
-    len = strlen(action);
+    len = STRLEN(action);
 
     if (len == 0 && item->name2 != NULL) {
 	action = item->name2;
@@ -472,9 +457,7 @@ menuitem_add(menu_t *menu, const char *name, const char *name2, const char *acti
 
     return item;
 }
-#endif
 
-#ifdef MENUBAR
 /*
  * search for the base starting menu for NAME.
  * return a pointer to the portion of NAME that remains
@@ -492,10 +475,10 @@ menu_find_base(menu_t **menu, char *path)
     if (path[0] == '\0')
 	return path;
 
-    if (strchr(path, '/') != NULL) {
+    if (STRCHR(path, '/') != NULL) {
 	register char  *p = path;
 
-	while ((p = strchr(p, '/')) != NULL) {
+	while ((p = STRCHR(p, '/')) != NULL) {
 	    p++;
 	    if (*p == '/')
 		path = p;
@@ -504,13 +487,13 @@ menu_find_base(menu_t **menu, char *path)
 	    path++;
 	    *menu = NULL;
 	}
-	while ((p = strchr(path, '/')) != NULL) {
+	while ((p = STRCHR(path, '/')) != NULL) {
 	    p[0] = '\0';
 	    if (path[0] == '\0')
 		return NULL;
-	    if (!strcmp(path, DOT)) {
+	    if (!STRCMP(path, DOT)) {
 	    /* nothing to do */
-	    } else if (!strcmp(path, DOTS)) {
+	    } else if (!STRCMP(path, DOTS)) {
 		if (*menu != NULL)
 		    *menu = (*menu)->parent;
 	    } else {
@@ -524,8 +507,8 @@ menu_find_base(menu_t **menu, char *path)
 	    path = (p + 1);
 	}
     }
-    if (!strcmp(path, DOTS)) {
-	path += strlen(DOTS);
+    if (!STRCMP(path, DOTS)) {
+	path += STRLEN(DOTS);
 	if (*menu != NULL)
 	    *menu = (*menu)->parent;
 	return path;
@@ -533,14 +516,14 @@ menu_find_base(menu_t **menu, char *path)
 /* find this menu */
     if (*menu == NULL) {
 	for (m = CurrentBar->tail; m != NULL; m = m->prev) {
-	    if (!strcmp(path, m->name))
+	    if (!STRCMP(path, m->name))
 		break;
 	}
     } else {
     /* find this menu */
 	for (item = (*menu)->tail; item != NULL; item = item->prev) {
 	    if (item->entry.type == MenuSubMenu
-		&& !strcmp(path, (item->entry.submenu.menu)->name)) {
+		&& !STRCMP(path, (item->entry.submenu.menu)->name)) {
 		m = (item->entry.submenu.menu);
 		break;
 	    }
@@ -548,13 +531,11 @@ menu_find_base(menu_t **menu, char *path)
     }
     if (m != NULL) {
 	*menu = m;
-	path += strlen(path);
+	path += STRLEN(path);
     }
     return path;
 }
-#endif
 
-#ifdef MENUBAR
 /*
  * delete this entire menu
  */
@@ -617,9 +598,7 @@ menu_delete(menu_t *menu)
 
     return parent;
 }
-#endif
 
-#ifdef MENUBAR
 /* INTPROTO */
 menu_t         *
 menu_add(menu_t *parent, char *path)
@@ -628,7 +607,7 @@ menu_add(menu_t *parent, char *path)
 
     assert(CurrentBar != NULL);
 
-    if (strchr(path, '/') != NULL) {
+    if (STRCHR(path, '/') != NULL) {
 	register char  *p;
 
 	if (path[0] == '/') {
@@ -636,7 +615,7 @@ menu_add(menu_t *parent, char *path)
 	    path++;
 	    parent = NULL;
 	}
-	while ((p = strchr(path, '/')) != NULL) {
+	while ((p = STRCHR(path, '/')) != NULL) {
 	    p[0] = '\0';
 	    if (path[0] == '\0')
 		return NULL;
@@ -645,10 +624,10 @@ menu_add(menu_t *parent, char *path)
 	    path = (p + 1);
 	}
     }
-    if (!strcmp(path, DOTS))
+    if (!STRCMP(path, DOTS))
 	return (parent != NULL ? parent->parent : parent);
 
-    if (!strcmp(path, DOT) || path[0] == '\0')
+    if (!STRCMP(path, DOT) || path[0] == '\0')
 	return parent;
 
 /* allocate a new menu */
@@ -657,7 +636,7 @@ menu_add(menu_t *parent, char *path)
 
     menu->width = 0;
     menu->parent = parent;
-    menu->len = strlen(path);
+    menu->len = STRLEN(path);
     menu->name = MALLOC((menu->len + 1));
     if (menu->name == NULL) {
 	FREE(menu);
@@ -698,9 +677,7 @@ menu_add(menu_t *parent, char *path)
 
     return menu;
 }
-#endif
 
-#ifdef MENUBAR
 /* INTPROTO */
 void
 drawbox_menubar(int x, int len, int state)
@@ -734,9 +711,7 @@ drawbox_menubar(int x, int len, int state)
     Draw_Shadow(menuBar.win, top, bot,
 		x, 0, len, menuBar_TotalHeight());
 }
-#endif
 
-#ifdef MENUBAR
 /* INTPROTO */
 void
 drawtriangle(int x, int y, int state)
@@ -768,9 +743,7 @@ drawtriangle(int x, int y, int state)
 
     Draw_Triangle(ActiveMenu->win, top, bot, x, y, w, 'r');
 }
-#endif
 
-#ifdef MENUBAR
 /* INTPROTO */
 void
 drawbox_menuitem(int y, int state)
@@ -801,10 +774,8 @@ drawbox_menuitem(int y, int state)
 		HEIGHT_TEXT + 2 * SHADOW);
     XFlush(Xdisplay);
 }
-#endif
 
 #ifdef DEBUG_MENU_LAYOUT
-#ifdef MENUBAR
 /* INTPROTO */
 void
 print_menu_ancestors(menu_t *menu)
@@ -831,9 +802,7 @@ print_menu_ancestors(menu_t *menu)
     fprintf(stderr, "\n");
     print_menu_ancestors(menu->parent);
 }
-#endif
 
-#ifdef MENUBAR
 /* INTPROTO */
 void
 print_menu_descendants(menu_t *menu)
@@ -873,9 +842,7 @@ print_menu_descendants(menu_t *menu)
     fprintf(stderr, "\n");
 }
 #endif
-#endif
 
-#ifdef MENUBAR
 /* pop up/down the current menu and redraw the menuBar button */
 /* INTPROTO */
 void
@@ -979,7 +946,7 @@ menu_show(void)
 
 		menu->x = (x1 < 0 ? 0 : x1);
 		menu->y = (y1 < 0 ? 0 : y1);
-	    } else if (item->name2 && !strcmp(name, item->name2))
+	    } else if (item->name2 && !STRCMP(name, item->name2))
 		name = NULL;
 
 	    if (len && name)
@@ -1003,9 +970,7 @@ menu_show(void)
 	y += h;
     }
 }
-#endif
 
-#ifdef MENUBAR
 /* INTPROTO */
 void
 menu_display(void (*update) (void))
@@ -1022,27 +987,21 @@ menu_display(void (*update) (void))
     ActiveMenu = ActiveMenu->parent;
     update();
 }
-#endif
 
-#ifdef MENUBAR
 /* INTPROTO */
 void
 menu_hide_all(void)
 {
     menu_display(menu_hide_all);
 }
-#endif
 
-#ifdef MENUBAR
 /* INTPROTO */
 void
 menu_hide(void)
 {
     menu_display(menu_show);
 }
-#endif
 
-#ifdef MENUBAR
 /* INTPROTO */
 void
 menu_clear(menu_t *menu)
@@ -1060,9 +1019,7 @@ menu_clear(menu_t *menu)
 	menu->width = 0;
     }
 }
-#endif
 
-#ifdef MENUBAR
 /* INTPROTO */
 void
 menubar_clear(void)
@@ -1086,10 +1043,8 @@ menubar_clear(void)
     }
     ActiveMenu = NULL;
 }
-#endif
 
 #if (MENUBAR_MAX > 1)
-#ifdef MENUBAR
 /* find if menu already exists */
 /* INTPROTO */
 bar_t          *
@@ -1103,9 +1058,9 @@ menubar_find(const char *name)
     if (bar == NULL || name == NULL)
 	return NULL;
 
-    if (strlen(name) && strcmp(name, "*")) {
+    if (STRLEN(name) && STRCMP(name, "*")) {
 	do {
-	    if (!strcmp(bar->name, name)) {
+	    if (!STRCMP(bar->name, name)) {
 #ifdef DEBUG_MENUBAR_STACKING
 		fprintf(stderr, " found!\n");
 #endif
@@ -1122,9 +1077,7 @@ menubar_find(const char *name)
 
     return bar;
 }
-#endif
 
-#ifdef MENUBAR
 /* INTPROTO */
 int
 menubar_push(const char *name)
@@ -1191,9 +1144,7 @@ menubar_push(const char *name)
 
     return ret;
 }
-#endif
 
-#ifdef MENUBAR
 /* switch to a menu called NAME and remove it */
 /* INTPROTO */
 void
@@ -1227,11 +1178,9 @@ menubar_remove(const char *name)
 	    CurrentBar = prev;
 	}
     }
-    while (CurrentBar && !strcmp(name, "*"));
+    while (CurrentBar && !STRCMP(name, "*"));
 }
-#endif
 
-#ifdef MENUBAR
 /* INTPROTO */
 void
 action_decode(FILE *fp, action_t *act)
@@ -1303,9 +1252,7 @@ action_decode(FILE *fp, action_t *act)
     }
     fprintf(fp, "\n");
 }
-#endif
 
-#ifdef MENUBAR
 /* INTPROTO */
 void
 menu_dump(FILE *fp, menu_t *menu)
@@ -1326,13 +1273,13 @@ menu_dump(FILE *fp, menu_t *menu)
 
 	case MenuLabel:
 	    fprintf(fp, "{%s}\n",
-		    (strlen(item->name) ? item->name : "-"));
+		    (STRLEN(item->name) ? item->name : "-"));
 	    break;
 
 	case MenuTerminalAction:
 	case MenuAction:
 	    fprintf(fp, "{%s}", item->name);
-	    if (item->name2 != NULL && strlen(item->name2))
+	    if (item->name2 != NULL && STRLEN(item->name2))
 		fprintf(fp, "{%s}", item->name2);
 	    fprintf(fp, "\t");
 	    action_decode(fp, &(item->entry.action));
@@ -1342,9 +1289,7 @@ menu_dump(FILE *fp, menu_t *menu)
 
     fprintf(fp, (menu->parent ? "../\n" : "/\n\n"));
 }
-#endif
 
-#ifdef MENUBAR
 /* INTPROTO */
 void
 menubar_dump(FILE *fp)
@@ -1358,7 +1303,7 @@ menubar_dump(FILE *fp)
 
     fprintf(fp,
 	    "# " APL_SUBCLASS " (%s)  Pid: %u\n# Date: %s\n\n",
-	    rs[Rs_name], (unsigned int)getpid(), ctime(&t));
+	    rs.name, (unsigned int)getpid(), ctime(&t));
 
 /* dump in reverse order */
     bar = CurrentBar->prev;
@@ -1390,7 +1335,6 @@ menubar_dump(FILE *fp)
     }
     while (bar != CurrentBar->prev);
 }
-#endif
 #endif				/* (MENUBAR_MAX > 1) */
 
 /*
@@ -1412,7 +1356,6 @@ menubar_dump(FILE *fp)
 void
 menubar_read(const char *filename)
 {
-#ifdef MENUBAR
 /* read in a menu from a file */
     FILE           *fp;
     char            buffer[256];
@@ -1428,7 +1371,7 @@ menubar_read(const char *filename)
 
 #if (MENUBAR_MAX > 1)
 /* semi-colon delimited */
-    if ((tag = strchr(filename, ';')) != NULL) {
+    if ((tag = STRCHR(filename, ';')) != NULL) {
 	tag++;
 	if (*tag == '\0')
 	    tag = NULL;
@@ -1509,7 +1452,6 @@ menubar_read(const char *filename)
     }
 
     fclose(fp);
-#endif
 }
 
 /*
@@ -1519,7 +1461,6 @@ menubar_read(const char *filename)
 void
 menubar_dispatch(char *str)
 {
-#ifdef MENUBAR
     static menu_t  *BuildMenu = NULL;	/* the menu currently being built */
     int             n, cmd;
     char           *path, *name, *name2;
@@ -1559,7 +1500,7 @@ menubar_dispatch(char *str)
 	    if (str[0] == ':') {	/* [:command:] */
 		do {
 		    next++;
-		    if ((next = strchr(next, ':')) == NULL)
+		    if ((next = STRCHR(next, ':')) == NULL)
 			return;	/* parse error */
 		}
 		while (next[1] != ']');
@@ -1567,7 +1508,7 @@ menubar_dispatch(char *str)
 		*next = '\0';
 		next += 2;
 	    } else {
-		if ((next = strchr(next, ']')) == NULL)
+		if ((next = STRCHR(next, ']')) == NULL)
 		    return;	/* parse error */
 	    /* remove and skip ']' */
 		*next = '\0';
@@ -1584,14 +1525,14 @@ menubar_dispatch(char *str)
 		menu_readonly = saved;
 	    }
 	/* these ones don't require menu stacking */
-	    else if (!strcmp(str, "clear")) {
+	    else if (!STRCMP(str, "clear")) {
 		menubar_clear();
-	    } else if (!strcmp(str, "done") || Str_match(str, "done:")) {
+	    } else if (!STRCMP(str, "done") || Str_match(str, "done:")) {
 		menu_readonly = 1;
-	    } else if (!strcmp(str, "show")) {
+	    } else if (!STRCMP(str, "show")) {
 		map_menuBar(1);
 		menu_readonly = 1;
-	    } else if (!strcmp(str, "hide")) {
+	    } else if (!STRCMP(str, "hide")) {
 		map_menuBar(0);
 		menu_readonly = 1;
 	    } else if ((n = Str_match(str, "read:")) != 0) {
@@ -1602,7 +1543,7 @@ menubar_dispatch(char *str)
 		str += n;
 		if (CurrentBar != NULL && !menu_readonly) {
 		    if (*str) {
-			name = REALLOC(CurrentBar->title, strlen(str) + 1);
+			name = REALLOC(CurrentBar->title, STRLEN(str) + 1);
 			if (name != NULL) {
 			    STRCPY(name, str);
 			    CurrentBar->title = name;
@@ -1652,7 +1593,7 @@ menubar_dispatch(char *str)
 
 		if (CurrentBar != NULL)
 		    menu_readonly = 0;	/* allow menu build commands */
-	    } else if (!strcmp(str, "dump")) {
+	    } else if (!STRCMP(str, "dump")) {
 	    /* dump current menubars to a file */
 		FILE           *fp;
 
@@ -1667,17 +1608,17 @@ menubar_dispatch(char *str)
 		    menubar_dump(fp);
 		    fclose(fp);
 		}
-	    } else if (!strcmp(str, "next")) {
+	    } else if (!STRCMP(str, "next")) {
 		if (CurrentBar) {
 		    CurrentBar = CurrentBar->next;
 		    menu_readonly = 1;
 		}
-	    } else if (!strcmp(str, "prev")) {
+	    } else if (!STRCMP(str, "prev")) {
 		if (CurrentBar) {
 		    CurrentBar = CurrentBar->prev;
 		    menu_readonly = 1;
 		}
-	    } else if (!strcmp(str, "swap")) {
+	    } else if (!STRCMP(str, "swap")) {
 	    /* swap the top 2 menus */
 		if (CurrentBar) {
 		    bar_t          *prev = CurrentBar->prev;
@@ -1728,8 +1669,8 @@ menubar_dispatch(char *str)
 	name2 = NULL;
     /* parse STR, allow spaces inside (name)  */
 	if (path[0] != '\0') {
-	    name = strchr(path, MENUITEM_BEG);
-	    str = strchr(path, MENUITEM_END);
+	    name = STRCHR(path, MENUITEM_BEG);
+	    str = STRCHR(path, MENUITEM_END);
 	    if (name != NULL || str != NULL) {
 		if (name == NULL || str == NULL || str <= (name + 1)
 		    || (name > path && name[-1] != '/')) {
@@ -1738,7 +1679,7 @@ menubar_dispatch(char *str)
 		}
 		if (str[1] == MENUITEM_BEG) {
 		    name2 = (str + 2);
-		    str = strchr(name2, MENUITEM_END);
+		    str = STRCHR(name2, MENUITEM_END);
 
 		    if (str == NULL) {
 			print_error("menu error <%s>\n", path);
@@ -1772,13 +1713,13 @@ menubar_dispatch(char *str)
 		int             len;
 
 		path = menu_find_base(&BuildMenu, path);
-		len = strlen(path);
+		len = STRLEN(path);
 
 	    /* don't allow menus called `*' */
 		if (path[0] == '*') {
 		    menu_clear(BuildMenu);
 		    break;
-		} else if (len >= 2 && !strcmp((path + len - 2), "/*")) {
+		} else if (len >= 2 && !STRCMP((path + len - 2), "/*")) {
 		    path[len - 2] = '\0';
 		}
 		if (path[0] != '\0')
@@ -1786,12 +1727,12 @@ menubar_dispatch(char *str)
 	    }
 	    if (name != NULL && name[0] != '\0')
 		menuitem_add(BuildMenu,
-			     (strcmp(name, SEPARATOR_NAME) ? name : ""),
+			     (STRCMP(name, SEPARATOR_NAME) ? name : ""),
 			     name2, str);
 	    break;
 
 	case '-':		/* delete menu entry */
-	    if (!strcmp(path, "/*") && (name == NULL || name[0] == '\0')) {
+	    if (!STRCMP(path, "/*") && (name == NULL || name[0] == '\0')) {
 		menubar_clear();
 		BuildMenu = NULL;
 		menubar_expose();
@@ -1801,13 +1742,13 @@ menubar_dispatch(char *str)
 		menu_t         *menu = BuildMenu;
 
 		path = menu_find_base(&menu, path);
-		len = strlen(path);
+		len = STRLEN(path);
 
 	    /* submenu called `*' clears all menu items */
 		if (path[0] == '*') {
 		    menu_clear(menu);
 		    break;	/* done */
-		} else if (len >= 2 && !strcmp(&path[len - 2], "/*")) {
+		} else if (len >= 2 && !STRCMP(&path[len - 2], "/*")) {
 		/* done */
 		    break;
 		} else if (path[0] != '\0') {
@@ -1824,7 +1765,7 @@ menubar_dispatch(char *str)
 		    menuitem_t     *item;
 
 		    item = menuitem_find(BuildMenu,
-				         strcmp(name, SEPARATOR_NAME) ? name
+				         STRCMP(name, SEPARATOR_NAME) ? name
 								      : "");
 
 		    if (item != NULL && item->entry.type != MenuSubMenu) {
@@ -1846,10 +1787,8 @@ menubar_dispatch(char *str)
 	}
 	break;
     }
-#endif
 }
 
-#ifdef MENUBAR
 /* INTPROTO */
 void
 draw_Arrows(int name, int state)
@@ -1889,13 +1828,11 @@ draw_Arrows(int name, int state)
     }
     XFlush(Xdisplay);
 }
-#endif
 
 /* EXTPROTO */
 void
 menubar_expose(void)
 {
-#ifdef MENUBAR
     menu_t         *menu;
     int             x;
 
@@ -1988,7 +1925,7 @@ menubar_expose(void)
 		str++;
 		switch (str[0]) {
 		case 'n':
-		    s = rs[Rs_name];
+		    s = rs.name;
 		    break;	/* resource name */
 		case 'v':
 		    s = VERSION;
@@ -2017,14 +1954,12 @@ menubar_expose(void)
 			menuBar_height() - SHADOW,
 			title, len);
     }
-#endif
 }
 
 /* EXTPROTO */
 int
 menubar_mapping(int map)
 {
-#ifdef MENUBAR
     int             change = 0;
 
     if (map && !menubar_visible()) {
@@ -2042,10 +1977,8 @@ menubar_mapping(int map)
 	menubar_expose();
 
     return change;
-#endif
 }
 
-#ifdef MENUBAR
 /* INTPROTO */
 int
 menu_select(XButtonEvent *ev)
@@ -2177,9 +2110,7 @@ menu_select(XButtonEvent *ev)
     }
     return 0;
 }
-#endif
 
-#ifdef MENUBAR
 /* INTPROTO */
 void
 menubar_select(XButtonEvent *ev)
@@ -2259,7 +2190,6 @@ menubar_select(XButtonEvent *ev)
 	break;
     }
 }
-#endif
 
 /*
  * general dispatch routine,
@@ -2269,7 +2199,6 @@ menubar_select(XButtonEvent *ev)
 void
 menubar_control(XButtonEvent *ev)
 {
-#ifdef MENUBAR
     switch (ev->type) {
     case ButtonPress:
 	if (ev->button == Button1)
@@ -2304,24 +2233,20 @@ menubar_control(XButtonEvent *ev)
 	}
 	break;
     }
-#endif
 }
 
 /* EXTPROTO */
 void
 map_menuBar(int map)
 {
-#ifdef MENUBAR
     if (menubar_mapping(map))
 	resize_all_windows();
-#endif
 }
 
 /* EXTPROTO */
 void
 create_menuBar(Cursor cursor)
 {
-#ifdef MENUBAR
 /* menuBar: size doesn't matter */
     menuBar.win = XCreateSimpleWindow(Xdisplay, TermWin.parent[0],
 				      0, 0,
@@ -2334,16 +2259,13 @@ create_menuBar(Cursor cursor)
 		 (ExposureMask | ButtonPressMask | ButtonReleaseMask |
 		  Button1MotionMask));
 
-#endif
 }
 
 /* EXTPROTO */
 void
 Resize_menuBar(int x, int y, unsigned int width, unsigned int height)
 {
-#ifdef MENUBAR
     XMoveResizeWindow(Xdisplay, menuBar.win, x, y, width, height);
-#endif
 }
-
+#endif
 /*----------------------- end-of-file (C source) -----------------------*/
