@@ -1,12 +1,12 @@
 /*--------------------------------*-C-*---------------------------------*
  * File:	misc.c
  *----------------------------------------------------------------------*
- * $Id: misc.c,v 1.18 1998/11/23 08:10:27 mason Exp $
+ * $Id: misc.c,v 1.31 1999/12/13 04:53:59 mason Exp $
  *
  * All portions of code are copyright by their respective author/s.
  * Copyright (C) 1996      mj olesen <olesen@me.QueensU.CA> Queen's Univ at Kingston
  * Copyright (C) 1997,1998 Oezguer Kesim <kesim@math.fu-berlin.de>
- * Copyright (C) 1998      Geoff Wing <gcw@pobox.com>
+ * Copyright (C) 1998,1999 Geoff Wing <gcw@pobox.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,17 +23,17 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *----------------------------------------------------------------------*/
 
+#include "../config.h"		/* NECESSARY */
 #include "rxvt.h"		/* NECESSARY */
 #include "misc.intpro"		/* PROTOS for internal routines */
 
-/*----------------------------------------------------------------------*/
 /* EXTPROTO */
-const char     *
-my_basename(const char *str)
+char           *
+r_basename(const char *str)
 {
-    const char     *base = strrchr(str, '/');
+    char           *base = STRRCHR(str, '/');
 
-    return (base ? base + 1 : str);
+    return (char *)(base ? base + 1 : str);
 }
 
 /*
@@ -58,20 +58,15 @@ print_error(const char *fmt,...)
  * No Match
  *      return: 0
  * Match
- *      return: strlen (S2)
+ *      return: STRLEN (S2)
  */
 /* EXTPROTO */
 int
 Str_match(const char *s1, const char *s2)
 {
-    int             n;
+    int             n = STRLEN(s2);
 
-    if (s1 == NULL || s2 == NULL)
-	return 0;
-    for (n = 0; *s2; n++)
-	if (*s1++ != *s2++)
-	    return 0;
-    return n;
+    return ((STRNCMP(s1, s2, n) == 0) ? n : 0);
 }
 
 /* EXTPROTO */
@@ -150,53 +145,37 @@ Str_escaped(char *str)
     d = s = str;
 
     if (*s == 'M' && s[1] == '-') {
-    /* Emacs convenience, replace leading `M-..' with `\E..' */
+	/* Emacs convenience, replace leading `M-..' with `\E..' */
 	*d++ = '\033';
 	s += 2;
 	if (toupper(*s) == 'X')
-	/* append carriage-return for `M-xcommand' */
+	    /* append carriage-return for `M-xcommand' */
 	    for (*d++ = 'x', append = '\r', s++; isspace(*s); s++) ;
     }
-    for ( ; (ch = *s++); ) {
+    for (; (ch = *s++);) {
 	if (ch == '\\') {
-	    switch ((ch = *s++)) {
-	    case '0':	/* octal */
-	    case '1':	/* octal */
-	    case '2':	/* octal */
-	    case '3':	/* octal */
-	    case '4':	/* octal */
-	    case '5':	/* octal */
-	    case '6':	/* octal */
-	    case '7':	/* octal */
+	    ch = *s++;
+	    if (ch >= '0' && ch <= '7') {	/* octal */
 		num = ch - '0';
 		for (i = 0; i < 2; i++, s++) {
 		    ch = *s;
-		    if (ch < '0' && ch > '7')
+		    if (ch < '0' || ch > '7')
 			break;
 		    num = num * 8 + ch - '0';
 		}
-		ch = (char) num;
-		break;
-	    case 'a':	/* bell */
-	        ch = 007;
-	        break;
-	    case 'b':	/* backspace */
-	        ch = '\b';
-	        break;
-	    case 'E':	/* escape */
-	    case 'e':
-	        ch = 033;
-	        break;
-	    case 'n':	/* newline */
-	        ch = '\n';
-	        break;
-	    case 'r':	/* carriage-return */
-	        ch = '\r';
-	        break;
-	    case 't':	/* tab */
-	        ch = '\t';
-	        break;
-	    }
+		ch = (char)num;
+	    } else if (ch == 'a')
+		ch = 007;	/* bell */
+	    else if (ch == 'b')
+		ch = '\b';	/* backspace */
+	    else if (ch == 'E' || ch == 'e')
+		ch = 033;	/* escape */
+	    else if (ch == 'n')
+		ch = '\n';	/* newline */
+	    else if (ch == 'r')
+		ch = '\r';	/* carriage-return */
+	    else if (ch == 't')
+		ch = '\t';	/* tab */
 	} else if (ch == '^') {
 	    ch = *s++;
 	    ch = toupper(ch);
@@ -236,16 +215,16 @@ File_search_path(const char *pathlist, const char *file, const char *ext)
 {
     int             maxpath, len;
     const char     *p, *path;
-    char	    name[256];
+    char            name[256];
 
     if (!access(file, R_OK))	/* found (plain name) in current directory */
 	return strdup(file);
 
 /* semi-colon delimited */
-    if ((p = strchr(file, ';')))
+    if ((p = STRCHR(file, ';')))
 	len = (p - file);
     else
-	len = strlen(file);
+	len = STRLEN(file);
 
 #ifdef DEBUG_SEARCH_PATH
     getcwd(name, sizeof(name));
@@ -254,27 +233,27 @@ File_search_path(const char *pathlist, const char *file, const char *ext)
 #endif
 
 /* leave room for an extra '/' and trailing '\0' */
-    maxpath = sizeof(name) - (len + (ext ? strlen(ext) : 0) + 2);
+    maxpath = sizeof(name) - (len + (ext ? STRLEN(ext) : 0) + 2);
     if (maxpath <= 0)
 	return NULL;
 
 /* check if we can find it now */
-    strncpy(name, file, len);
+    STRNCPY(name, file, len);
     name[len] = '\0';
 
     if (!access(name, R_OK))
 	return strdup(name);
     if (ext) {
-	strcat(name, ext);
+	STRCAT(name, ext);
 	if (!access(name, R_OK))
 	    return strdup(name);
     }
     for (path = pathlist; path != NULL && *path != '\0'; path = p) {
 	int             n;
 
-    /* colon delimited */
-	if ((p = strchr(path, ':')) == NULL)
-	    p = strchr(path, '\0');
+	/* colon delimited */
+	if ((p = STRCHR(path, ':')) == NULL)
+	    p = STRCHR(path, '\0');
 
 	n = (p - path);
 	if (*p != '\0')
@@ -285,12 +264,12 @@ File_search_path(const char *pathlist, const char *file, const char *ext)
 	    if (name[n - 1] != '/')
 		name[n++] = '/';
 	    name[n] = '\0';
-	    strncat(name, file, len);
+	    STRNCAT(name, file, len);
 
 	    if (!access(name, R_OK))
 		return strdup(name);
 	    if (ext) {
-		strcat(name, ext);
+		STRCAT(name, ext);
 		if (!access(name, R_OK))
 		    return strdup(name);
 	    }
@@ -309,7 +288,7 @@ File_find(const char *file, const char *ext)
 	return NULL;
 
 /* search environment variables here too */
-    if ((f = File_search_path(rs[Rs_path], file, ext)) == NULL)
+    if ((f = File_search_path(rs.path, file, ext)) == NULL)
 #ifdef PATH_ENV
 	if ((f = File_search_path(getenv(PATH_ENV), file, ext)) == NULL)
 #endif
@@ -331,6 +310,7 @@ File_find(const char *file, const char *ext)
 /*
  * Draw top/left and bottom/right border shadows around windows
  */
+#if (! defined(NEXT_SCROLLBAR) && ! defined(XTERM_SCROLLBAR)) || defined(MENUBAR)
 /* EXTPROTO */
 void
 Draw_Shadow(Window win, GC topShadow, GC botShadow, int x, int y, int w, int h)
@@ -343,8 +323,8 @@ Draw_Shadow(Window win, GC topShadow, GC botShadow, int x, int y, int w, int h)
     x1 = x;
     y1 = y;
     for (; shadow-- > 0; x1++, y1++, w1--, h1--) {
-        XDrawLine(Xdisplay, win, topShadow, x1, y1, w1, y1);
-        XDrawLine(Xdisplay, win, topShadow, x1, y1, x1, h1);
+	XDrawLine(Xdisplay, win, topShadow, x1, y1, w1, y1);
+	XDrawLine(Xdisplay, win, topShadow, x1, y1, x1, h1);
     }
 
     shadow = (w == 0 || h == 0) ? 1 : SHADOW;
@@ -353,12 +333,14 @@ Draw_Shadow(Window win, GC topShadow, GC botShadow, int x, int y, int w, int h)
     x1 = x + 1;
     y1 = y + 1;
     for (; shadow-- > 0; x1++, y1++, w1--, h1--) {
-        XDrawLine(Xdisplay, win, botShadow, w1, h1, w1, y1);
-        XDrawLine(Xdisplay, win, botShadow, w1, h1, x1, h1);
+	XDrawLine(Xdisplay, win, botShadow, w1, h1, w1, y1);
+	XDrawLine(Xdisplay, win, botShadow, w1, h1, x1, h1);
     }
 }
+#endif
 
 /* button shapes */
+#ifdef MENUBAR
 /* EXTPROTO */
 void
 Draw_Triangle(Window win, GC topShadow, GC botShadow, int x, int y, int w, int type)
@@ -397,4 +379,5 @@ Draw_Triangle(Window win, GC topShadow, GC botShadow, int x, int y, int w, int t
 #endif
     }
 }
+#endif
 /*----------------------- end-of-file (C source) -----------------------*/
