@@ -1,35 +1,10 @@
 /*
- * $Id: rxvt.h,v 1.40.2.6 1999/07/17 09:43:31 mason Exp $
+ * $Id: rxvt.h,v 1.65 1999/12/18 07:41:47 mason Exp $
  */
 
 #ifndef _RXVT_H			/* include once only */
 #define _RXVT_H
 
-#include "../config.h"
-
-/* sort out conflicts in feature.h */
-#undef  MULTICHAR_SET		/* a glyph is only ever defined by 1 char */
-#ifdef KANJI
-# define MULTICHAR_SET		/* a glyph is defined by 1 or 2 chars     */
-# undef ZH			/* remove Chinese big5 support            */
-# undef ZHCN			/* remove Chinese gb support	          */
-# undef GREEK_SUPPORT		/* Kanji/Greek together is too weird      */
-# undef DEFINE_XTERM_COLOR	/* since kterm-color doesn't exist?       */
-#endif
-#ifdef ZH 
-# define MULTICHAR_SET		/* a glyph is defined by 1 or 2 chars     */
-# undef KANJI			/* can't put Chinese/Kanji together       */
-# undef ZHCN
-# undef GREEK_SUPPORT		
-# undef DEFINE_XTERM_COLOR
-#endif
-#ifdef ZHCN
-# define MULTICHAR_SET		/* a glyph is defined by 1 or 2 chars     */
-# undef KANJI
-# undef ZH
-# undef GREEK_SUPPORT
-#endif
-
 #include "feature.h"
 
 /*
@@ -80,18 +55,20 @@
 #ifdef HAVE_STDLIB_H
 # include <stdlib.h>
 #endif
-#ifdef HAVE_STRING_H
-# include <string.h>
-#endif
-#ifdef HAVE_FCNTL_H
-# include <fcntl.h>
-#endif
 #ifdef HAVE_SYS_TYPES_H
 # include <sys/types.h>
 #endif
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif
+#ifdef NO_STRINGS
+# ifdef HAVE_STRING_H
+#  include <string.h>
+# endif
+#endif
+#ifdef HAVE_FCNTL_H
+# include <fcntl.h>
+#endif
 #ifdef HAVE_UTIL_H
 # include <util.h>
 #endif
@@ -135,6 +112,12 @@ typedef char           *XPointer;
 # include "grkelot.h"
 #endif
 
+#ifndef STDIN_FILENO
+# define STDIN_FILENO	0
+# define STDOUT_FILENO	1
+# define STDERR_FILENO	2
+#endif
+
 /*
  *****************************************************************************
  * STRUCTURES AND TYPEDEFS
@@ -163,7 +146,7 @@ typedef struct {
 #endif
 
 #ifdef TRANSPARENT
-# define KNOW_PARENTS		4
+# define KNOW_PARENTS		6
 #else
 # define KNOW_PARENTS		1
 #endif
@@ -174,9 +157,13 @@ typedef struct {
 		    fwidth,	/* font width [pixels]                      */
 		    fheight,	/* font height [pixels]                     */
 		    fprop,	/* font is proportional                     */
+		    bprop,	/* treat bold font as proportional          */
+		    mprop,	/* treat multichar font as proportional     */
 		    ncol, nrow,	/* window size [characters]                 */
 		    focus,	/* window has focus                         */
 		    mapped,	/* window state mapped?                     */
+		    int_bwidth,
+		    ext_bwidth,
 		    saveLines;	/* number of lines that fit in scrollback   */
     u_int16_t       nscrolled,	/* number of line actually scrolled         */
 		    view_start;	/* scrollback view starts here              */
@@ -190,7 +177,7 @@ typedef struct {
 #ifdef MULTICHAR_SET
     XFontStruct    *mfont;	/* Multichar font structure                 */
 #endif
-    XFontSet       fontset;
+    XFontSet        fontset;
 #ifdef XPM_BACKGROUND
     Pixmap          pixmap;
 #ifdef XPM_BUFFERING
@@ -216,15 +203,6 @@ typedef struct grcmd_t {
     struct grcmd_t *next;
 } grcmd_t;
 
-typedef struct grwin_t {
-    Window          win;
-    int             x, y;
-    unsigned int    w, h;
-    short           screen;
-    grcmd_t        *graphics;
-    struct grwin_t *prev, *next;
-} grwin_t;
-
 typedef struct {
     int16_t         row, col;
 } row_col_t;
@@ -299,9 +277,6 @@ typedef struct {
 #define scrollBar_esc		30
 #define menuBar_margin		2	/* margin below text */
 
-/* gap between text and window edges (could be configurable) */
-#define TermWin_internalBorder	2
-
 /* width of scrollBar, menuBar shadow, must be 1 or 2 */
 #ifdef HALFSHADOW
 # define SHADOW 1
@@ -312,6 +287,65 @@ typedef struct {
 #ifdef NEXT_SCROLLBAR
 # undef SB_WIDTH
 # define SB_WIDTH		19
+# define SB_PADDING		1
+# define SB_BORDER_WIDTH	1
+# define SB_BEVEL_WIDTH_UPPER_LEFT	1
+# define SB_BEVEL_WIDTH_LOWER_RIGHT	2
+# define SB_LEFT_PADDING	(SB_PADDING + SB_BORDER_WIDTH)
+# define SB_MARGIN_SPACE	(SB_PADDING * 2)
+# define SB_BUTTON_WIDTH	(SB_WIDTH - SB_MARGIN_SPACE - SB_BORDER_WIDTH)
+# define SB_BUTTON_HEIGHT	(SB_BUTTON_WIDTH)
+# define SB_BUTTON_SINGLE_HEIGHT	(SB_BUTTON_HEIGHT + SB_PADDING)
+# define SB_BUTTON_BOTH_HEIGHT		(SB_BUTTON_SINGLE_HEIGHT * 2)
+# define SB_BUTTON_TOTAL_HEIGHT		(SB_BUTTON_BOTH_HEIGHT + SB_PADDING)
+# define SB_BUTTON_BEVEL_X	(SB_LEFT_PADDING)
+# define SB_BUTTON_FACE_X	(SB_BUTTON_BEVEL_X + SB_BEVEL_WIDTH_UPPER_LEFT)
+# define SB_THUMB_MIN_HEIGHT	(SB_BUTTON_WIDTH - (SB_PADDING * 2))
+ /*
+  *    +-------------+
+  *    |             | <---< SB_PADDING
+  *    | ::::::::::: |
+  *    | ::::::::::: |
+  *   '''''''''''''''''
+  *   ,,,,,,,,,,,,,,,,,
+  *    | ::::::::::: |
+  *    | ::::::::::: |
+  *    |  +---------------< SB_BEVEL_WIDTH_UPPER_LEFT
+  *    |  | :::::::: |
+  *    |  V :::: vv-------< SB_BEVEL_WIDTH_LOWER_RIGHT
+  *    | +---------+ |
+  *    | | ......%%| |
+  *    | | ......%%| |
+  *    | | ..()..%%| |
+  *    | | ......%%| |
+  *    | | %%%%%%%%| |
+  *    | +---------+ | <.........................
+  *    |             | <---< SB_PADDING         :
+  *    | +---------+ | <-+..........            :---< SB_BUTTON_TOTAL_HEIGHT
+  *    | | ......%%| |   |         :            :
+  *    | | ../\..%%| |   |---< SB_BUTTON_HEIGHT :
+  *    | | %%%%%%%%| |   |         :            :
+  *    | +---------+ | <-+         :            :
+  *    |             |             :            :
+  *    | +---------+ | <-+         :---< SB_BUTTON_BOTH_HEIGHT
+  *    | | ......%%| |   |         :            :
+  *    | | ..\/..%%| |   |         :            :
+  *    | | %%%%%%%%| |   |---< SB_BUTTON_SINGLE_HEIGHT
+  *    | +---------+ |   |         :            :
+  *    |             |   |         :            :
+  *    +-------------+ <-+.........:............:
+  *    ^^|_________| :
+  *    ||     |      :
+  *    ||     +---< SB_BUTTON_WIDTH
+  *    ||            :
+  *    |+------< SB_PADDING
+  *    |:            :
+  *    +----< SB_BORDER_WIDTH
+  *     :            :
+  *     :............:
+  *           |
+  *           +---< SB_WIDTH
+  */
 #else
 # ifdef XTERM_SCROLLBAR
 #  undef  SB_WIDTH
@@ -388,8 +422,12 @@ enum {
 # define RS_multi1		0x20000000u	/* multibyte 1st byte */
 # define RS_multi2		(RS_multi0|RS_multi1)	/* multibyte 2nd byte */
 # define RS_multiMask		(RS_multi0|RS_multi1)	/* multibyte mask */
+# define IS_MULTI1(r)		(((r) & RS_multiMask) == RS_multi1)
+# define IS_MULTI2(r)		(((r) & RS_multiMask) == RS_multi2)
 #else
 # define RS_multiMask		0
+# define IS_MULTI1(r)		(0)
+# define IS_MULTI2(r)		(0)
 #endif
 
 #define RS_attrMask		(RS_baseattrMask|RS_fontMask|RS_multiMask)
@@ -428,6 +466,7 @@ enum {
 #define XTerm_Pixmap		20	/* new bg pixmap */
 #define XTerm_restoreFG		39	/* change default fg color */
 #define XTerm_restoreBG		49	/* change default bg color */
+#define XTerm_dumpscreen	55	/* dump scrollback and all of screen */
 
 #define restoreFG		39	/* restore default fg color */
 #define restoreBG		49	/* restore default bg color */
@@ -488,78 +527,80 @@ enum colour_list {
 #define DEFAULT_RSTYLE		(RS_None | (Color_fg) | (Color_bg<<5))
 
 /*
- * This resource list should match xdefaults.c
- * - though not necessarily in order
+ * Resource list
  */
-enum Rs_resource_list {
-    Rs_display_name = 0, 
-    Rs_term_name,
-    Rs_geometry,
-    Rs_reverseVideo,
-    Rs_color,
-    Rs_font = Rs_color + NRS_COLORS,
-    Rs_iconName = Rs_font + NFONTS,
+typedef struct {
+    const char     *display_name,
+		   *term_name,
+		   *iconName,
+		   *geometry,
+		   *reverseVideo,
+		   *color[NRS_COLORS],
+		   *font[NFONTS],
 #ifdef MULTICHAR_SET
-    Rs_mfont,
-    Rs_multichar_encoding = Rs_mfont + NFONTS,
+		   *mfont[NFONTS],
+		   *multichar_encoding,
 #endif
-    Rs_name,
-    Rs_title,
+		   *name,
+		   *title,
 #if defined (XPM_BACKGROUND) || (MENUBAR_MAX)
-    Rs_path,
+		   *path,
 #endif
 #ifdef XPM_BACKGROUND
-    Rs_backgroundPixmap,
+		   *backgroundPixmap,
 #endif
 #if (MENUBAR_MAX)
-    Rs_menu,
+		   *menu,
 #endif
 #ifndef NO_BOLDFONT
-    Rs_boldFont,
+		   *boldFont,
 #endif
 #ifdef GREEK_SUPPORT
-    Rs_greek_keyboard,
+		   *greek_keyboard,
 #endif
-    Rs_loginShell,
-    Rs_scrollBar,
-    Rs_scrollBar_right,
-    Rs_scrollBar_floating,
-    Rs_scrollTtyOutput,
-    Rs_scrollKeypress,
-    Rs_saveLines,
-    Rs_utmpInhibit,
-    Rs_visualBell,
+ 		   *loginShell,
+ 		   *scrollBar,
+ 		   *scrollBar_right,
+ 		   *scrollBar_floating,
+ 		   *scrollTtyOutput,
+ 		   *scrollKeypress,
+ 		   *saveLines,
+ 		   *utmpInhibit,
+ 		   *visualBell,
 #if ! defined(NO_MAPALERT) && defined(MAPALERT_OPTION)
-    Rs_mapAlert,
+		   *mapAlert,
 #endif
 #ifdef META8_OPTION
-    Rs_meta8,
+		   *meta8,
 #endif
 #ifndef NO_BACKSPACE_KEY
-    Rs_backspace_key,
+		   *backspace_key,
 #endif
 #ifndef NO_DELETE_KEY
-    Rs_delete_key,
+		   *delete_key,
 #endif
-    Rs_selectstyle,
+		   *selectstyle,
 #ifdef PRINTPIPE
-    Rs_print_pipe,
+		   *print_pipe,
 #endif
 #ifdef USE_XIM
-    Rs_preeditType,
-    Rs_inputMethod,
+		   *preeditType,
+		   *inputMethod,
 #endif
 #if defined (HOTKEY_CTRL) || defined (HOTKEY_META)
-    Rs_bigfont_key,
-    Rs_smallfont_key,
+		   *bigfont_key,
+		   *smallfont_key,
 #endif
 #ifdef TRANSPARENT
-    Rs_transparent,
+		   *transparent,
 #endif
-    Rs_cutchars,
-    Rs_modifier,
-    TOTAL_RS
-} ;
+#ifndef NO_FRILLS
+		   *ext_bwidth,
+		   *int_bwidth,
+#endif
+		   *cutchars,
+		   *modifier;
+} resource_list;
 
 /*
  * number of graphics points
@@ -573,31 +614,39 @@ enum Rs_resource_list {
  * MACRO DEFINES
  *****************************************************************************
  */
-#define STRCPY(a, b)		strcpy ((char *)(a), (const char *) (b))
-#define STRNCPY(a, b, c)	strncpy ((char *)(a), (const char *) (b), (c))
-#ifdef NO_RMEMSET
-# define MEMSET(a, b, c)	memset ((a), (b), (c))
-#else
-# define MEMSET(a, b, c)	rmemset ((a), (b), (intp_t) (c))
-#endif
+#define MEMSET(x, y, z)		memset((x), (y), (z))
+#define MEMCPY(x, y, z)		memcpy((void *)(x), (const void *)(y), (z))
+#define MEMMOVE(x, y, z)	memmove((void *)(x), (const void *)(y), (z))
+#define STRCASECMP(x, y)	strcasecmp((x), (y))
+#define STRNCASECMP(x, y, z)	strncasecmp((x), (y), (z))
+#define STRCPY(x, y)		strcpy((char *)(x), (const char *)(y))
+#define STRNCPY(x, y, z)	strncpy((char *)(x), (const char *)(y), (z))
+#define STRCMP(x, y)		strcmp((const char *)(x), (const char *)(y))
+#define STRNCMP(x, y, z)	strncmp((const char *)(x), (const char *)(y), (z))
+#define STRCAT(x, y)		strcat((char *)(x), (const char *)(y))
+#define STRNCAT(x, y, z)	strncat((char *)(x), (const char *)(y), (z))
+#define STRDUP(x)		strdup((const char *)(x))
+#define STRLEN(x)		strlen((const char *)(x))
+#define STRCHR(x, y)		strchr((const char *)(x), (int)(y))
+#define STRRCHR(x, y)		strrchr((const char *)(x), (int)(y))
 
-#define MALLOC(sz)		malloc (sz)
-#define CALLOC(type, n)		calloc ((n), sizeof(type))
-#define REALLOC(mem, sz)	((mem) ? realloc ((mem), (sz)) : malloc(sz))
-#define FREE(ptr)		free (ptr)
+#define MALLOC(sz)		malloc(sz)
+#define CALLOC(n, sz)		calloc((n), (sz))
+#define REALLOC(mem, sz)	((mem) ? realloc((mem), (sz)) : malloc(sz))
+#define FREE(ptr)		free(ptr)
 
 /* convert pixel dimensions to row/column values */
-#define Pixel2Col(x)		Pixel2Width((x) - TermWin_internalBorder)
-#define Pixel2Row(y)		Pixel2Height((y) - TermWin_internalBorder)
+#define Pixel2Col(x)		Pixel2Width((x) - TermWin.int_bwidth)
+#define Pixel2Row(y)		Pixel2Height((y) - TermWin.int_bwidth)
 #define Pixel2Width(x)		((x) / TermWin.fwidth)
 #define Pixel2Height(y)		((y) / TermWin.fheight)
-#define Col2Pixel(col)		(Width2Pixel(col) + TermWin_internalBorder)
-#define Row2Pixel(row)		(Height2Pixel(row) + TermWin_internalBorder)
+#define Col2Pixel(col)		(Width2Pixel(col) + TermWin.int_bwidth)
+#define Row2Pixel(row)		(Height2Pixel(row) + TermWin.int_bwidth)
 #define Width2Pixel(n)		((n) * TermWin.fwidth)
 #define Height2Pixel(n)		((n) * TermWin.fheight)
 
-#define TermWin_TotalWidth()	(TermWin.width  + 2 * TermWin_internalBorder)
-#define TermWin_TotalHeight()	(TermWin.height + 2 * TermWin_internalBorder)
+#define TermWin_TotalWidth()	(TermWin.width  + 2 * TermWin.int_bwidth)
+#define TermWin_TotalHeight()	(TermWin.height + 2 * TermWin.int_bwidth)
 
 #define Xscreen			DefaultScreen(Xdisplay)
 #define Xroot			DefaultRootWindow(Xdisplay)
@@ -631,7 +680,7 @@ enum Rs_resource_list {
 # define scrollbar_upButton(y)	((y) > scrollBar.end \
 				 && (y) <= scrollbar_dnval())
 # define scrollbar_dnButton(y)	((y) > scrollbar_dnval())
-# define SCROLL_MINHEIGHT	15
+# define SCROLL_MINHEIGHT	SB_THUMB_MIN_HEIGHT
 #else
 # define scrollbar_upButton(y)	((y) < scrollBar.beg)
 # define scrollbar_dnButton(y)	((y) > scrollBar.end)
@@ -662,6 +711,10 @@ enum Rs_resource_list {
 # define XPMClearArea(a, b, c, d, e, f, g)
 #endif
 
+#ifndef STRICT_FONT_CHECKING
+# define get_fontwidest(font)	((font)->max_bounds.width)
+#endif
+
 #define Gr_ButtonPress(x,y)	Gr_ButtonReport ('P',(x),(y))
 #define Gr_ButtonRelease(x,y)	Gr_ButtonReport ('R',(x),(y))
 /*
@@ -702,7 +755,7 @@ EXTERN int              display_is_local;
 #endif
 EXTERN short		want_refresh;
 
-EXTERN const char      *rs[TOTAL_RS];
+EXTERN resource_list    rs;
 
 #ifndef NO_BACKSPACE_KEY
 EXTERN const char      *key_backspace;
@@ -735,6 +788,44 @@ EXTERN KeySym		ks_smallfont;
 #endif
 #include "protos.h"
 
+#ifndef RXVT_GRAPHICS		/* sync functions with graphics.extpro */
+# define Gr_ButtonReport(but, x, y)
+# define Gr_do_graphics(cmd, nargs, args, text)
+# define Gr_scroll(count)
+# define Gr_ClearScreen()
+# define Gr_ChangeScreen()
+# define Gr_expose(win)
+# define Gr_Resize(w, h)
+# define Gr_reset()
+# define Gr_Displayed()		(0)
+#endif
+
+#ifndef MENUBAR			/* sync functions with menubar.extpro */
+# define menubar_read(filename)
+# define menubar_dispatch(str)
+# define menubar_expose()
+# define menubar_mapping(map)	(0)
+# define menubar_control(ev)
+# define map_menuBar(map)
+# define create_menuBar(cursor)
+# define Resize_menuBar(x, y, width, height)
+#endif
+
+#ifndef XPM_BACKGROUND		/* sync functions with xpm.extpro */
+# define scale_pixmap(geom)	(0)
+# define resize_pixmap()
+# define set_bgPixmap(file)	(0)
+#endif
+
+#ifndef GREEK_SUPPORT		/* sync functions with grkelot.extpro */
+# define greek_init()
+# define greek_end()
+# define greek_reset()
+# define greek_setmode(greek_mode)
+# define greek_getmode()	(0)
+# define greek_xlat(s, num_chars)	(0)
+#endif
+
 #ifdef DEBUG_MALLOC
 # include "dmalloc.h"		/* This comes last */
 #endif
