@@ -1,5 +1,5 @@
 /*
- * $Id: screen.h,v 1.8 1998/11/25 16:34:13 mason Exp $
+ * $Id: screen.h,v 1.13 1999/08/05 06:48:53 mason Exp $
  */
 
 #ifndef _SCREEN_H		/* include once only */
@@ -42,7 +42,7 @@ typedef unsigned char text_t;
  * TermWin.nscrolled:
  *              Actual number of lines we've used of the scrollback buffer
  *              0 <= TermWin.nscrolled <= TermWin.saveLines
- * TermWin.view_start:  
+ * TermWin.view_start:
  *              Offset back into the scrollback buffer for out current view
  *              0 <= TermWin.view_start <= TermWin.nscrolled
  *
@@ -61,15 +61,12 @@ typedef struct {
                     bscroll,	/* bottom of settable scroll region          */
 		    charset;	/* character set number [0..3]               */
     unsigned int    flags;	/* see below                                 */
+    row_col_t	    s_cur;	/* saved cursor position                     */
+    int16_t	    s_charset;	/* saved character set number [0..3]         */
+    char            s_charset_char;
+    rend_t          s_rstyle;	/* saved rendition style                     */
 } screen_t;
 
-typedef struct {
-    row_col_t	    cur;	/* cursor position                           */
-    int16_t	    charset;	/* character set number [0..3]               */
-    char            charset_char;
-    rend_t          rstyle;	/* rendition style                           */
-} save_t;
-
 typedef struct {
     unsigned char  *text;	/* selected text                             */
     int             len;	/* length of selected text                   */
@@ -99,6 +96,21 @@ typedef enum {
 #define Screen_WrapNext		(1<<4)	/* need to wrap for next char?       */
 #define Screen_DefaultFlags	(Screen_VisibleCursor|Screen_Autowrap)
 
+/* ------------------------------------------------------------------------- */
+
+typedef enum {
+    EUCJ, SJIS,			/* Japanese KANJI methods                    */
+    BIG5, CNS,			/* Chinese BIG5 methods: CNS not implemented */
+    GB,				/* Chinese GB method                         */
+    EUCKR			/* Korean method                             */
+} ENC_METHOD;
+
+typedef struct {
+    short          method;
+    void (*func)(unsigned char *, int);
+    char          *name;
+} KNOWN_ENCODINGS;
+
 /* ------------------------------------------------------------------------- *
  *                             MODULE VARIABLES                              *
  * ------------------------------------------------------------------------- */
@@ -118,8 +130,11 @@ EXTSCR int16_t    *buf_tlen;
 EXTSCR char       *tabs;	/* a 1 for a location with a tab-stop */
 EXTSCR screen_t    screen;
 EXTSCR screen_t    swap;
-EXTSCR save_t      save;
 EXTSCR selection_t selection;
 EXTSCR sstyle_t    selection_style;
+#ifdef MULTICHAR_SET
+EXTSCR ENC_METHOD  encoding_method;
+#endif
+
 
 #endif				/* repeat inclusion protection */
