@@ -1,7 +1,7 @@
 /*--------------------------------*-C-*---------------------------------*
  * File:	scrollbar.c
  *----------------------------------------------------------------------*
- * $Id: scrollbar.c,v 1.19.2.3 1999/09/20 18:39:41 mason Exp $
+ * $Id: scrollbar.c,v 1.26 1999/12/18 04:44:23 mason Exp $
  *
  * Copyright (C) 1997,1998 mj olesen <olesen@me.QueensU.CA>
  * Copyright (C) 1998      Alfredo K. Kojima <kojima@windowmaker.org>
@@ -22,6 +22,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *----------------------------------------------------------------------*/
 
+#include "../config.h"		/* NECESSARY */
 #include "rxvt.h"		/* NECESSARY */
 #include "scrollbar.intpro"	/* PROTOS for internal routines */
 
@@ -345,7 +346,7 @@ int
 scrollbar_show(int update)
 {
     static int      scrollbar_len;	/* length of slider */
-    static int      last_top, last_bot, last_state;
+    static int      last_top, last_bot, last_state = -1;
 #ifndef NEXT_SCROLLBAR
     static short    sb_width;		/* old (drawn) values */
     int             xsb = 0;
@@ -604,8 +605,32 @@ map_scrollBar(int map)
 {
     if (scrollbar_mapping(map)) {
 	resize_all_windows();
-	scr_touch();
+	scr_touch(True);
     }
 }
 
+/* EXTPROTO */
+void
+Resize_scrollBar(void)
+{
+    unsigned int    x, xsb;
+
+    xsb = (SB_WIDTH + 2 * sb_shadow);
+    x = (Options & Opt_scrollBar_right) ? (szHint.width - xsb) : 0;
+    XMoveResizeWindow(Xdisplay, scrollBar.win, x, 0, xsb, szHint.height);
+
+    scrollBar.beg = 0;
+    scrollBar.end = szHint.height;
+#ifndef XTERM_SCROLLBAR
+/* arrows are as high as wide - leave 1 pixel gap */
+# ifdef NEXT_SCROLLBAR
+    scrollBar.end -= SB_BUTTON_TOTAL_HEIGHT + SB_PADDING;
+# else
+    scrollBar.beg += (SB_WIDTH + 1) + sb_shadow;
+    scrollBar.end -= (SB_WIDTH + 1) + sb_shadow;
+# endif
+#endif
+    scrollbar_show(1);
+}
+
 /*----------------------- end-of-file (C source) -----------------------*/
