@@ -1,7 +1,7 @@
 /*--------------------------------*-C-*---------------------------------*
  * File:	xdefaults.c
  *----------------------------------------------------------------------*
- * $Id: xdefaults.c,v 1.34.2.4 2000/03/09 09:41:36 mason Exp $
+ * $Id: xdefaults.c,v 1.51 1999/12/18 06:51:24 mason Exp $
  *
  * All portions of code are copyright by their respective author/s.
  * Copyright (C) 1994      Robert Nation <nation@rocket.sanders.lockheed.com>
@@ -25,20 +25,17 @@
 /*----------------------------------------------------------------------*
  * get resources from ~/.Xdefaults or ~/.Xresources with the memory-saving
  * default or with XGetDefault() (#define USE_XGETDEFAULT)
- *
- * Coding style:
- *	resource strings are indicated by an `Rs_' prefix followed by
- *	the resource name.
- *	eg, `Rs_saveLines' is the resource string corresponding to
- *	    the `saveLines' resource
  *----------------------------------------------------------------------*/
 
+#include "../config.h"		/* NECESSARY */
 #include "rxvt.h"		/* NECESSARY */
 #include "version.h"
 #include "xdefaults.intpro"	/* PROTOS for internal routines */
 
 /* #define DEBUG_RESOURCES */
 
+static const char *const xnames[2] = { ".Xdefaults", ".Xresources" };
+
 /*{{{ monolithic option/resource structure: */
 /*
  * `string' options MUST have a usage argument
@@ -47,28 +44,28 @@
  */
 
 /* INFO() - descriptive information only */
-#define INFO(opt, arg, desc)				\
-    {0, NULL, NULL, opt, arg, desc}
+#define INFO(opt, arg, desc)					\
+    {0, NULL, NULL, (opt), (arg), (desc)}
 
 /* STRG() - command-line option, with/without resource */
-#define STRG(p, kw, opt, arg, desc)			\
-    {0, &(rs[p]), kw, opt, arg, desc}
+#define STRG(rsp, kw, opt, arg, desc)				\
+    {0, &(rsp), (kw), (opt), (arg), (desc)}
 
 /* RSTRG() - resource/long-option */
-#define RSTRG(p, kw, arg)				\
-    {0, &(rs[p]), kw, NULL, arg, NULL}
+#define RSTRG(rsp, kw, arg)					\
+    {0, &(rsp), (kw), NULL, (arg), NULL}
 
 /* BOOL() - regular boolean `-/+' flag */
-#define BOOL(p, kw, opt, flag, desc)			\
-    {(Opt_Boolean|flag), &(rs[p]), kw, opt, NULL, desc}
+#define BOOL(rsp, kw, opt, flag, desc)				\
+    {(Opt_Boolean|(flag)), &(rsp), (kw), (opt), NULL, (desc)}
 
 /* SWCH() - `-' flag */
-#define SWCH(opt, flag, desc)				\
-    {(flag), NULL, NULL, opt, NULL, desc}
+#define SWCH(opt, flag, desc)					\
+    {(flag), NULL, NULL, (opt), NULL, (desc)}
 
 /* convenient macros */
-#define optList_strlen(i)						\
-    (optList[i].flag ? 0 : (optList[i].arg ? strlen (optList[i].arg) : 1))
+#define optList_STRLEN(i)						\
+    (optList[i].flag ? 0 : (optList[i].arg ? STRLEN (optList[i].arg) : 1))
 #define optList_isBool(i)						\
     (optList[i].flag & Opt_Boolean)
 #define optList_isReverse(i)						\
@@ -84,182 +81,189 @@ static const struct {
     const char     *arg;	/* argument */
     const char     *desc;	/* description */
 } optList[] = {
-    STRG(Rs_display_name, NULL, "d", NULL, NULL),	/* short form */
-    STRG(Rs_display_name, NULL, "display", "string", "X server to contact"),
-    STRG(Rs_term_name, "termName", "tn", "string",
-         "value of the TERM environment variable"),
-    STRG(Rs_geometry, NULL, "g", NULL, NULL),	/* short form */
-    STRG(Rs_geometry, "geometry", "geometry", "geometry",
-         "size (in characters) and position"),
+    STRG(rs.display_name, NULL, "d", NULL, NULL),	/* short form */
+    STRG(rs.display_name, NULL, "display", "string", "X server to contact"),
+    STRG(rs.term_name, "termName", "tn", "string",
+	 "value of the TERM environment variable"),
+    STRG(rs.geometry, NULL, "g", NULL, NULL),	/* short form */
+    STRG(rs.geometry, "geometry", "geometry", "geometry",
+	 "size (in characters) and position"),
     SWCH("C", Opt_console, "intercept console messages"),
     SWCH("iconic", Opt_iconic, "start iconic"),
     SWCH("ic", Opt_iconic, NULL),	/* short form */
-    BOOL(Rs_reverseVideo, "reverseVideo", "rv", Opt_reverseVideo,
-         "reverse video"),
-    BOOL(Rs_loginShell, "loginShell", "ls", Opt_loginShell, "login shell"),
-    BOOL(Rs_scrollBar, "scrollBar", "sb", Opt_scrollBar, "scrollbar"),
-    BOOL(Rs_scrollBar_right, "scrollBar_right", "sr", Opt_scrollBar_right,
-         "scrollbar right"),
-    BOOL(Rs_scrollBar_floating, "scrollBar_floating", "st",
-         Opt_scrollBar_floating, "scrollbar without a trough"),
-    BOOL(Rs_scrollTtyOutput, "scrollTtyOutput", NULL,
-         Opt_scrollTtyOutput, NULL),
-    BOOL(Rs_scrollTtyOutput, NULL, "si", Opt_Reverse | Opt_scrollTtyOutput,
+    BOOL(rs.reverseVideo, "reverseVideo", "rv", Opt_reverseVideo,
+	 "reverse video"),
+    BOOL(rs.loginShell, "loginShell", "ls", Opt_loginShell, "login shell"),
+    BOOL(rs.scrollBar, "scrollBar", "sb", Opt_scrollBar, "scrollbar"),
+    BOOL(rs.scrollBar_right, "scrollBar_right", "sr", Opt_scrollBar_right,
+	 "scrollbar right"),
+    BOOL(rs.scrollBar_floating, "scrollBar_floating", "st",
+	 Opt_scrollBar_floating, "scrollbar without a trough"),
+    BOOL(rs.scrollTtyOutput, "scrollTtyOutput", NULL, Opt_scrollTtyOutput,
+	 NULL),
+    SWCH("si", Opt_Boolean | Opt_Reverse | Opt_scrollTtyOutput,
 	 "scroll-on-tty-output inhibit"),
-    BOOL(Rs_scrollKeypress, "scrollTtyKeypress", "sk",
-         Opt_scrollKeypress, "scroll-on-keypress"),
+    BOOL(rs.scrollKeypress, "scrollTtyKeypress", "sk", Opt_scrollKeypress,
+	 "scroll-on-keypress"),
 #ifdef TRANSPARENT
-    BOOL(Rs_transparent, "inheritPixmap", "ip",
-	 Opt_transparent, "inherit parent pixmap"),
+    BOOL(rs.transparent, "inheritPixmap", "ip", Opt_transparent,
+	 "inherit parent pixmap"),
     SWCH("tr", Opt_transparent, NULL),
 #endif
-    BOOL(Rs_utmpInhibit, "utmpInhibit", "ut", Opt_utmpInhibit,
-         "utmp inhibit"),
-    BOOL(Rs_visualBell, "visualBell", "vb", Opt_visualBell, "visual bell"),
-#if ! defined(NO_MAPALERT) && defined(MAPALERT_OPTION)
-    BOOL(Rs_mapAlert, "mapAlert", NULL, Opt_mapAlert, NULL),
+    BOOL(rs.utmpInhibit, "utmpInhibit", "ut", Opt_utmpInhibit, "utmp inhibit"),
+#ifndef NO_BELL
+    BOOL(rs.visualBell, "visualBell", "vb", Opt_visualBell, "visual bell"),
+# if ! defined(NO_MAPALERT) && defined(MAPALERT_OPTION)
+    BOOL(rs.mapAlert, "mapAlert", NULL, Opt_mapAlert, NULL),
+# endif
 #endif
 #ifdef META8_OPTION
-    BOOL(Rs_meta8, "meta8", NULL, Opt_meta8, NULL),
+    BOOL(rs.meta8, "meta8", NULL, Opt_meta8, NULL),
 #endif
-    STRG(Rs_color + Color_bg, "background", "bg", "color", "background color"),
-    STRG(Rs_color + Color_fg, "foreground", "fg", "color", "foreground color"),
-    RSTRG(Rs_color + minCOLOR + 0, "color0", "color"),
-    RSTRG(Rs_color + minCOLOR + 1, "color1", "color"),
-    RSTRG(Rs_color + minCOLOR + 2, "color2", "color"),
-    RSTRG(Rs_color + minCOLOR + 3, "color3", "color"),
-    RSTRG(Rs_color + minCOLOR + 4, "color4", "color"),
-    RSTRG(Rs_color + minCOLOR + 5, "color5", "color"),
-    RSTRG(Rs_color + minCOLOR + 6, "color6", "color"),
-    RSTRG(Rs_color + minCOLOR + 7, "color7", "color"),
+    STRG(rs.color[Color_bg], "background", "bg", "color", "background color"),
+    STRG(rs.color[Color_fg], "foreground", "fg", "color", "foreground color"),
+    RSTRG(rs.color[minCOLOR + 0], "color0", "color"),
+    RSTRG(rs.color[minCOLOR + 1], "color1", "color"),
+    RSTRG(rs.color[minCOLOR + 2], "color2", "color"),
+    RSTRG(rs.color[minCOLOR + 3], "color3", "color"),
+    RSTRG(rs.color[minCOLOR + 4], "color4", "color"),
+    RSTRG(rs.color[minCOLOR + 5], "color5", "color"),
+    RSTRG(rs.color[minCOLOR + 6], "color6", "color"),
+    RSTRG(rs.color[minCOLOR + 7], "color7", "color"),
 #ifndef NO_BRIGHTCOLOR
-    RSTRG(Rs_color + minBrightCOLOR + 0, "color8", "color"),
-    RSTRG(Rs_color + minBrightCOLOR + 1, "color9", "color"),
-    RSTRG(Rs_color + minBrightCOLOR + 2, "color10", "color"),
-    RSTRG(Rs_color + minBrightCOLOR + 3, "color11", "color"),
-    RSTRG(Rs_color + minBrightCOLOR + 4, "color12", "color"),
-    RSTRG(Rs_color + minBrightCOLOR + 5, "color13", "color"),
-    RSTRG(Rs_color + minBrightCOLOR + 6, "color14", "color"),
-    RSTRG(Rs_color + minBrightCOLOR + 7, "color15", "color"),
+    RSTRG(rs.color[minBrightCOLOR + 0], "color8", "color"),
+    RSTRG(rs.color[minBrightCOLOR + 1], "color9", "color"),
+    RSTRG(rs.color[minBrightCOLOR + 2], "color10", "color"),
+    RSTRG(rs.color[minBrightCOLOR + 3], "color11", "color"),
+    RSTRG(rs.color[minBrightCOLOR + 4], "color12", "color"),
+    RSTRG(rs.color[minBrightCOLOR + 5], "color13", "color"),
+    RSTRG(rs.color[minBrightCOLOR + 6], "color14", "color"),
+    RSTRG(rs.color[minBrightCOLOR + 7], "color15", "color"),
 #endif				/* NO_BRIGHTCOLOR */
 #ifndef NO_BOLDUNDERLINE
-    RSTRG(Rs_color + Color_BD, "colorBD", "color"),
-    RSTRG(Rs_color + Color_UL, "colorUL", "color"),
+    RSTRG(rs.color[Color_BD], "colorBD", "color"),
+    RSTRG(rs.color[Color_UL], "colorUL", "color"),
 #endif				/* NO_BOLDUNDERLINE */
 #ifdef KEEP_SCROLLCOLOR
-    RSTRG(Rs_color + Color_scroll, "scrollColor", "color"),
-    RSTRG(Rs_color + Color_trough, "troughColor", "color"),
+    RSTRG(rs.color[Color_scroll], "scrollColor", "color"),
+    RSTRG(rs.color[Color_trough], "troughColor", "color"),
 #endif				/* KEEP_SCROLLCOLOR */
 #if defined (XPM_BACKGROUND) || (MENUBAR_MAX)
-    RSTRG(Rs_path, "path", "search path"),
+    RSTRG(rs.path, "path", "search path"),
 #endif				/* defined (XPM_BACKGROUND) || (MENUBAR_MAX) */
 #ifdef XPM_BACKGROUND
-    STRG(Rs_backgroundPixmap, "backgroundPixmap",
-         "pixmap", "file[;geom]", "background pixmap"),
+    STRG(rs.backgroundPixmap, "backgroundPixmap", "pixmap", "file[;geom]",
+	 "background pixmap"),
 #endif				/* XPM_BACKGROUND */
 #if (MENUBAR_MAX)
-    RSTRG(Rs_menu, "menu", "name[;tag]"),
+    RSTRG(rs.menu, "menu", "name[;tag]"),
 #endif
 #ifndef NO_BOLDFONT
-    STRG(Rs_boldFont, "boldFont", "fb", "fontname", "bold text font"),
+    STRG(rs.boldFont, "boldFont", "fb", "fontname", "bold text font"),
 #endif
-    STRG(Rs_font, "font", "fn", "fontname", "normal text font"),
+    STRG(rs.font[0], "font", "fn", "fontname", "normal text font"),
 /* fonts: command-line option = resource name */
 #if NFONTS > 1
-    RSTRG(Rs_font + 1, "font1", "fontname"),
+    RSTRG(rs.font[1], "font1", "fontname"),
 #endif
 #if NFONTS > 2
-    RSTRG(Rs_font + 2, "font2", "fontname"),
+    RSTRG(rs.font[2], "font2", "fontname"),
 #endif
 #if NFONTS > 3
-    RSTRG(Rs_font + 3, "font3", "fontname"),
+    RSTRG(rs.font[3], "font3", "fontname"),
 #endif
 #if NFONTS > 4
-    RSTRG(Rs_font + 4, "font4", "fontname"),
+    RSTRG(rs.font[4], "font4", "fontname"),
 #endif
 #if NFONTS > 5
-    RSTRG(Rs_font + 5, "font5", "fontname"),
+    RSTRG(rs.font[5], "font5", "fontname"),
 #endif
 #if NFONTS > 6
-    RSTRG(Rs_font + 6, "font6", "fontname"),
+    RSTRG(rs.font[6], "font6", "fontname"),
 #endif
 #if NFONTS > 7
-    RSTRG(Rs_font + 7, "font7", "fontname"),
+    RSTRG(rs.font[7], "font7", "fontname"),
 #endif
 #ifdef MULTICHAR_SET
-    STRG(Rs_mfont, "mfont", "fm", "fontname", "multichar font"),
-
+    STRG(rs.mfont[0], "mfont", "fm", "fontname", "multichar font"),
 /* fonts: command-line option = resource name */
 # if NFONTS > 1
-    RSTRG(Rs_mfont + 1, "mfont1", "fontname"),
+    RSTRG(rs.mfont[1], "mfont1", "fontname"),
 # endif
 # if NFONTS > 2
-    RSTRG(Rs_mfont + 2, "mfont2", "fontname"),
+    RSTRG(rs.mfont[2], "mfont2", "fontname"),
 # endif
 # if NFONTS > 3
-    RSTRG(Rs_mfont + 3, "mfont3", "fontname"),
+    RSTRG(rs.mfont[3], "mfont3", "fontname"),
 # endif
 # if NFONTS > 4
-    RSTRG(Rs_mfont + 4, "mfont4", "fontname"),
+    RSTRG(rs.mfont[4], "mfont4", "fontname"),
 # endif
 # if NFONTS > 5
-    RSTRG(Rs_mfont + 5, "mfont5", "fontname"),
+    RSTRG(rs.mfont[5], "mfont5", "fontname"),
 # endif
 # if NFONTS > 6
-    RSTRG(Rs_mfont + 6, "mfont6", "fontname"),
+    RSTRG(rs.mfont[6], "mfont6", "fontname"),
 # endif
 # if NFONTS > 7
-    RSTRG(Rs_mfont + 7, "mfont7", "fontname"),
+    RSTRG(rs.mfont[7], "mfont7", "fontname"),
 # endif
 #endif				/* MULTICHAR_SET */
 #ifdef MULTICHAR_SET
-    STRG(Rs_multichar_encoding, "multichar_encoding", "km", "mode",
-         "multiple-character font encoding; mode = eucj | sjis | big5 | gb"),
+    STRG(rs.multichar_encoding, "multichar_encoding", "km", "mode",
+	 "multiple-character font encoding; mode = eucj | sjis | big5 | gb | kr"),
 #endif				/* MULTICHAR_SET */
 #ifdef USE_XIM
-    STRG(Rs_preeditType, "preeditType", "pt", "style",
-         "input style of input method; style = OverTheSpot | OffTheSpot | Root"),
-    STRG(Rs_inputMethod, "inputMethod", "im", "name",
-         "name of input method"),
-#endif                         /* USE_XIM */
+    STRG(rs.preeditType, "preeditType", "pt", "style",
+	 "input style of input method; style = OverTheSpot | OffTheSpot | Root"),
+    STRG(rs.inputMethod, "inputMethod", "im", "name", "name of input method"),
+#endif				/* USE_XIM */
 #ifdef GREEK_SUPPORT
-    STRG(Rs_greek_keyboard, "greek_keyboard", "grk", "mode",
-         "greek keyboard mapping; mode = iso | ibm"),
+    STRG(rs.greek_keyboard, "greek_keyboard", "grk", "mode",
+	 "greek keyboard mapping; mode = iso | ibm"),
 #endif
-    STRG(Rs_name, NULL, "name", "string",
-         "client instance, icon, and title strings"),
-    STRG(Rs_title, "title", "title", "string", "title name for window"),
-    STRG(Rs_title, NULL, "T", NULL, NULL),	/* short form */
-    STRG(Rs_iconName, "iconName", "n", "string", "icon name for window"),
+    STRG(rs.name, NULL, "name", "string",
+	 "client instance, icon, and title strings"),
+    STRG(rs.title, "title", "title", "string", "title name for window"),
+    STRG(rs.title, NULL, "T", NULL, NULL),	/* short form */
+    STRG(rs.iconName, "iconName", "n", "string", "icon name for window"),
 #ifndef NO_CURSORCOLOR
-    STRG(Rs_color + Color_cursor, "cursorColor", "cr", "color",
-         "cursor color"),
+    STRG(rs.color[Color_cursor], "cursorColor", "cr", "color", "cursor color"),
 /* command-line option = resource name */
-    RSTRG(Rs_color + Color_cursor2, "cursorColor2", "color"),
+    RSTRG(rs.color[Color_cursor2], "cursorColor2", "color"),
 #endif				/* NO_CURSORCOLOR */
-    STRG(Rs_color + Color_pointer, "pointerColor", "pr", "color",
-         "pointer color"),
-    STRG(Rs_color + Color_border, "borderColor", "bd", "color",
-         "border color"),
-    STRG(Rs_saveLines, "saveLines", "sl", "number",
-         "number of scrolled lines to save"),
+    STRG(rs.color[Color_pointer], "pointerColor", "pr", "color",
+	 "pointer color"),
+    STRG(rs.color[Color_border], "borderColor", "bd", "color",
+	 "border color"),
+    STRG(rs.saveLines, "saveLines", "sl", "number",
+	 "number of scrolled lines to save"),
+#ifndef NO_FRILLS
+    STRG(rs.ext_bwidth, "externalBorder", "w", "number",
+	 "external border in pixels"),
+    STRG(rs.ext_bwidth, NULL, "bw", NULL, NULL),
+    STRG(rs.ext_bwidth, NULL, "borderwidth", NULL, NULL),
+    STRG(rs.int_bwidth, "internalBorder", "b", "number",
+	 "internal border in pixels"),
+#endif
 #ifndef NO_BACKSPACE_KEY
-    RSTRG(Rs_backspace_key, "backspacekey", "string"),
+    RSTRG(rs.backspace_key, "backspacekey", "string"),
 #endif
 #ifndef NO_DELETE_KEY
-    RSTRG(Rs_delete_key, "deletekey", "string"),
+    RSTRG(rs.delete_key, "deletekey", "string"),
 #endif
-    RSTRG(Rs_selectstyle, "selectstyle", "string"),
+    RSTRG(rs.selectstyle, "selectstyle", "string"),
 #ifdef PRINTPIPE
-    RSTRG(Rs_print_pipe, "print-pipe", "string"),
+    RSTRG(rs.print_pipe, "print-pipe", "string"),
 #endif
 #if defined (HOTKEY_CTRL) || defined (HOTKEY_META)
-    RSTRG(Rs_bigfont_key, "bigfont_key", "keysym"),
-    RSTRG(Rs_smallfont_key, "smallfont_key", "keysym"),
+    RSTRG(rs.bigfont_key, "bigfont_key", "keysym"),
+    RSTRG(rs.smallfont_key, "smallfont_key", "keysym"),
 #endif
-    STRG(Rs_modifier, "modifier", "mod", "modifier", "alt | meta | hyper | super | mod1 | ... | mod5"),
+    STRG(rs.modifier, "modifier", "mod", "modifier",
+	 "alt | meta | hyper | super | mod1 | ... | mod5"),
 #ifdef CUTCHAR_RESOURCE
-    RSTRG(Rs_cutchars, "cutchars", "string"),
+    RSTRG(rs.cutchars, "cutchars", "string"),
 #endif				/* CUTCHAR_RESOURCE */
     INFO("e", "command arg ...", "command to execute")
 };
@@ -294,14 +298,11 @@ usage(int type)
 #ifdef MENUBAR
     fprintf(stderr, "menubar,");
 #endif
-#ifdef KANJI
-    fprintf(stderr, "Kanji,");
+#ifdef USE_XIM
+    fprintf(stderr, "XIM,");
 #endif
-#ifdef ZH
-    fprintf(stderr, "Chinese,");
-#endif
-#ifdef ZHCN
-    fprintf(stderr, "Chinese(GB),");
+#ifdef MULTICHAR_SET
+    fprintf(stderr, "multichar languages,");
 #endif
 #ifdef XTERM_SCROLLBAR
     fprintf(stderr, "XTerm-scrollbar,");
@@ -324,7 +325,7 @@ usage(int type)
 # ifdef USE_XGETDEFAULT
     fprintf(stderr, "XGetDefaults");
 # else
-    fprintf(stderr, ".Xdefaults");
+    fprintf(stderr, xnames[0]);
 # endif
 #endif
 
@@ -338,11 +339,11 @@ usage(int type)
 		int             len = 2;
 
 		if (!optList_isBool(i)) {
-		    len = optList_strlen(i);
+		    len = optList_STRLEN(i);
 		    if (len > 0)
 			len++;	/* account for space */
 		}
-		len += 4 + strlen(optList[i].opt);
+		len += 4 + STRLEN(optList[i].opt);
 
 		col += len;
 		if (col > 79) {	/* assume regular width */
@@ -353,7 +354,7 @@ usage(int type)
 		if (optList_isBool(i))
 		    fprintf(stderr, "/+");
 		fprintf(stderr, "%s", optList[i].opt);
-		if (optList_strlen(i))
+		if (optList_STRLEN(i))
 		    fprintf(stderr, " %s]", optList[i].arg);
 		else
 		    fprintf(stderr, "]");
@@ -363,17 +364,15 @@ usage(int type)
 	break;
 
     case 1:			/* full command-line listing */
-	fprintf(stderr,
-		" [options] [-e command args]\n\n"
+	fprintf(stderr, " [options] [-e command args]\n\n"
 		"where options include:\n");
 
 	for (i = 0; i < optList_size(); i++)
 	    if (optList[i].desc != NULL)
 		fprintf(stderr, "    %s%s %-*s%s%s\n",
 			(optList_isBool(i) ? "-/+" : "-"),
-			optList[i].opt,
-			(INDENT - strlen(optList[i].opt)
-			 + (optList_isBool(i) ? 0 : 2)),
+			optList[i].opt, (INDENT - STRLEN(optList[i].opt)
+					 + (optList_isBool(i) ? 0 : 2)),
 			(optList[i].arg ? optList[i].arg : ""),
 			(optList_isBool(i) ? "turn on/off " : ""),
 			optList[i].desc);
@@ -389,12 +388,12 @@ usage(int type)
 	    if (optList[i].kw != NULL)
 		fprintf(stderr, "    %s: %*s\n",
 			optList[i].kw,
-			(INDENT - strlen(optList[i].kw)),
+			(INDENT - STRLEN(optList[i].kw)),
 			(optList_isBool(i) ? "boolean" : optList[i].arg));
 
 #ifdef KEYSYM_RESOURCE
 	fprintf(stderr, "    " "keysym.sym" ": %*s\n",
-		(INDENT - strlen("keysym.sym")), "keysym");
+		(INDENT - sizeof("keysym.sym") + 1), "keysym");
 #endif
 	fprintf(stderr, "\n    -help to list options\n\n");
 	break;
@@ -402,12 +401,13 @@ usage(int type)
     exit(EXIT_FAILURE);
     /* NOTREACHED */
 }
+
 /*}}} */
 
 /*{{{ get command-line options before getting resources */
 /* EXTPROTO */
 void
-get_options(int argc, const char * const *argv)
+get_options(int argc, const char *const *argv)
 {
     int             i, bad_option = 0;
     static const char On[3] = "ON", Off[4] = "OFF";
@@ -415,7 +415,7 @@ get_options(int argc, const char * const *argv)
     for (i = 1; i < argc; i++) {
 	int             entry, longopt = 0;
 	const char     *flag, *opt;
-	
+
 	opt = argv[i];
 #ifdef DEBUG_RESOURCES
 	fprintf(stderr, "argv[%d] = %s: ", i, opt);
@@ -434,22 +434,22 @@ get_options(int argc, const char * const *argv)
 	    continue;
 	}
 
-	if (!strcmp(opt, "help"))
+	if (!STRCMP(opt, "help"))
 	    usage(longopt ? 2 : 1);
-	if (!strcmp(opt, "h"))
+	if (!STRCMP(opt, "h"))
 	    usage(0);
 
-    /* feature: always try to match long-options */
+	/* feature: always try to match long-options */
 	for (entry = 0; entry < optList_size(); entry++)
-	    if ((optList[entry].kw && !strcmp(opt, optList[entry].kw))
+	    if ((optList[entry].kw && !STRCMP(opt, optList[entry].kw))
 		|| (!longopt
-		    && optList[entry].opt && !strcmp(opt, optList[entry].opt)))
+		    && optList[entry].opt && !STRCMP(opt, optList[entry].opt)))
 		break;
 
 	if (entry < optList_size()) {
 	    if (optList_isReverse(entry))
 		flag = flag == On ? Off : On;
-	    if (optList_strlen(entry)) {	/* string value */
+	    if (optList_STRLEN(entry)) {	/* string value */
 		const char     *str = argv[++i];
 
 #ifdef DEBUG_RESOURCES
@@ -462,11 +462,11 @@ get_options(int argc, const char * const *argv)
 		    fprintf(stderr, "\"%s\"\n", str);
 #endif
 		    *(optList[entry].dp) = str;
-
-		/* special cases are handled in main.c:main() to allow
-		 * X resources to set these values before we settle for
-		 * default values
-		 */
+		    /*
+		     * special cases are handled in main.c:main() to allow
+		     * X resources to set these values before we settle for
+		     * default values
+		     */
 		}
 #ifdef DEBUG_RESOURCES
 		else
@@ -487,30 +487,28 @@ get_options(int argc, const char * const *argv)
 	    }
 	} else
 #ifdef KEYSYM_RESOURCE
-	/* if (!strncmp (opt, "keysym.", strlen ("keysym."))) */
+	    /* if (!STRNCMP(opt, "keysym.", sizeof("keysym.") - 1)) */
 	if (Str_match(opt, "keysym.")) {
 	    const char     *str = argv[++i];
 
-	/*
-	 * '7' is strlen("keysym.")
-	 */
 	    if (str != NULL)
-		parse_keysym(opt + 7, str);
+		parse_keysym(opt + sizeof("keysym.") - 1, str);
 	} else
 #endif
 	{
-	/* various old-style options, just ignore
-	 * Obsolete since about Jan 96,
-	 * so they can probably eventually be removed
-	 */
+	    /*
+	     * various old-style options, just ignore
+	     * Obsolete since about Jan 96,
+	     * so they can probably eventually be removed
+	     */
 	    const char     *msg = "bad";
 
 	    if (longopt) {
 		opt--;
 		bad_option = 1;
-	    } else if (!strcmp(opt, "7") || !strcmp(opt, "8")
+	    } else if (!STRCMP(opt, "7") || !STRCMP(opt, "8")
 #ifdef GREEK_SUPPORT
-	    /* obsolete 12 May 1996 (v2.17) */
+		       /* obsolete 12 May 1996 (v2.17) */
 		       || !Str_match(opt, "grk")
 #endif
 		)
@@ -525,29 +523,11 @@ get_options(int argc, const char * const *argv)
     if (bad_option)
 	usage(0);
 }
+
 /*}}} */
 
 #ifndef NO_RESOURCES
 /*----------------------------------------------------------------------*/
-/*{{{ string functions */
-/*
- * a replacement for strcasecmp() to avoid linking an entire library
- */
-/* INTPROTO */
-int
-my_strcasecmp(const char *s1, const char *s2)
-{
-    for ( /*nil */ ; (*s1 && *s2); s1++, s2++) {
-	register int    c1 = toupper(*s1);
-	register int    c2 = toupper(*s2);
-
-	if (c1 != c2)
-	    return (c1 - c2);
-    }
-    return (int)(*s1 - *s2);
-}
-
-/*}}} */
 
 # ifdef KEYSYM_RESOURCE
 /*
@@ -597,28 +577,27 @@ parse_keysym(const char *str, const char *arg)
 	if (str[0] == '0' && toupper(str[1]) == 'X')
 	    str += 2;
 	if (arg) {
-	    if (sscanf(str, (strchr(str, ':') ? "%x:" : "%x"), &sym) != 1)
+	    if (sscanf(str, (STRCHR(str, ':') ? "%x:" : "%x"), &sym) != 1)
 		return -1;
 	} else {
 	    if (sscanf(str, "%x:", &sym) != 1)
 		return -1;
 
-	/* cue to ':', it's there since sscanf() worked */
-	    STRNCPY(newargstr, strchr(str, ':') + 1, NEWARGLIM - 1);
+	    /* cue to ':', it's there since sscanf() worked */
+	    STRNCPY(newargstr, STRCHR(str, ':') + 1, NEWARGLIM - 1);
 	    newargstr[NEWARGLIM - 1] = '\0';
-	    newarg = Str_trim(newargstr);
+	    newarg = newargstr;
 	}
     } else {
-    /*
-     * convert keysym name to keysym number
-     */
+	/*
+	 * convert keysym name to keysym number
+	 */
 	STRNCPY(newargstr, str, NEWARGLIM - 1);
 	newargstr[NEWARGLIM - 1] = '\0';
 	if (arg == NULL) {
-	    if ((newarg = strchr(newargstr, ':')) == NULL)
+	    if ((newarg = STRCHR(newargstr, ':')) == NULL)
 		return -1;
 	    *newarg++ = '\0';	/* terminate keysym name */
-	    (void) Str_trim(newarg);
 	}
 	if ((sym = XStringToKeysym(newargstr)) == None)
 	    return -1;
@@ -635,17 +614,19 @@ parse_keysym(const char *str, const char *arg)
 	newargstr[NEWARGLIM - 1] = '\0';
 	newarg = newargstr;
     }
-    if (strlen(newargstr) == 0
-	|| (n = Str_escaped(newargstr)) == 0)
+    (void)Str_trim(newarg);
+    if (*newarg == '\0' || (n = Str_escaped(newarg)) == 0)
 	return -1;
     MIN_IT(n, 255);
     key_string = MALLOC((n + 1) * sizeof(char));
+
     key_string[0] = n;
     STRNCPY(key_string + 1, newarg, n);
-    KeySym_map[sym] = (unsigned char *) key_string;
+    KeySym_map[sym] = (unsigned char *)key_string;
 
     return 1;
 }
+
 # endif				/* KEYSYM_RESOURCE */
 
 # ifndef USE_XGETDEFAULT
@@ -662,7 +643,7 @@ get_xdefaults(FILE *stream, const char *name)
 
     if (stream == NULL)
 	return;
-    len = strlen(name);
+    len = STRLEN(name);
     while ((str = fgets(buffer, sizeof(buffer), stream)) != NULL) {
 	unsigned int    entry, n;
 
@@ -670,7 +651,7 @@ get_xdefaults(FILE *stream, const char *name)
 	    str++;		/* leading whitespace */
 
 	if ((str[len] != '*' && str[len] != '.')
-	    || (len && strncmp(str, name, len)))
+	    || (len && STRNCMP(str, name, len)))
 	    continue;
 	str += (len + 1);	/* skip `name*' or `name.' */
 
@@ -682,24 +663,24 @@ get_xdefaults(FILE *stream, const char *name)
 
 		if (kw == NULL)
 		    continue;
-		n = strlen(kw);
+		n = STRLEN(kw);
 		if (str[n] == ':' && Str_match(str, kw)) {
-		/* skip `keyword:' */
+		    /* skip `keyword:' */
 		    str += (n + 1);
-		    (void) Str_trim(str);
-		    n = strlen(str);
+		    (void)Str_trim(str);
+		    n = STRLEN(str);
 		    if (n && *(optList[entry].dp) == NULL) {
-		    /* not already set */
+			/* not already set */
 			int             s;
 			char           *p = MALLOC((n + 1) * sizeof(char));
 
 			STRCPY(p, str);
 			*(optList[entry].dp) = p;
 			if (optList_isBool(entry)) {
-			    s = my_strcasecmp(p, "TRUE") == 0
-				|| my_strcasecmp(p, "YES") == 0
-				|| my_strcasecmp(p, "ON") == 0
-				|| my_strcasecmp(p, "1") == 0;
+			    s = STRCASECMP(str, "TRUE") == 0
+				|| STRCASECMP(str, "YES") == 0
+				|| STRCASECMP(str, "ON") == 0
+				|| STRCASECMP(str, "1") == 0;
 			    if (optList_isReverse(entry))
 				s = !s;
 			    if (s)
@@ -714,6 +695,7 @@ get_xdefaults(FILE *stream, const char *name)
     }
     rewind(stream);
 }
+
 /*}}} */
 # endif				/* ! USE_XGETDEFAULT */
 #endif				/* NO_RESOURCES */
@@ -735,28 +717,51 @@ extract_resources(Display *display, const char *name)
     int             entry;
 
 #  ifdef XrmEnumOneLevel
+    int             i;
+    char           *displayResource, *ptr, *xe;
     XrmName         name_prefix[3];
     XrmClass        class_prefix[3];
-    char           *displayResource;
-    XrmDatabase     database;
-    char           *screenResource;
-    XrmDatabase     screenDatabase;
+    XrmDatabase     database, rdb1;
+    char            fname[1024];
 
-/*
- * Get screen-specific resources (X11R5) and merge into common resources.
- */
+    XrmInitialize();
     database = NULL;
-    screenDatabase = NULL;
+
+/* Get any Xserver defaults */
+
     displayResource = XResourceManagerString(display);
     if (displayResource != NULL)
 	database = XrmGetStringDatabase(displayResource);
-    screenResource = XScreenResourceString(DefaultScreenOfDisplay(display));
-    if (screenResource != NULL)
-	screenDatabase = XrmGetStringDatabase(screenResource);
-    XrmMergeDatabases(screenDatabase, &database);
+
+/* Add in ~/.Xdefaults & ~/.Xresources */
+
+    if ((ptr = (char *)getenv("HOME")) == NULL)
+	ptr = ".";
+
+    for (i = 0; i < (sizeof(xnames) / sizeof(xnames[0])); i++) {
+	sprintf(fname, "%-.*s/%s", sizeof(fname) - STRLEN(xnames[i]) - 2, ptr,
+		xnames[i]);
+	if ((rdb1 = XrmGetFileDatabase(fname)) != NULL)
+	    XrmMergeDatabases(rdb1, &database);
+    }
+
+/* Add in XENVIRONMENT file */
+
+    if ((xe = (char *)getenv("XENVIRONMENT")) != NULL
+	&& (rdb1 = XrmGetFileDatabase(xe)) != NULL)
+	XrmMergeDatabases(rdb1, &database);
+
+/* Add in Rxvt file */
+
+    if ((rdb1 = XrmGetFileDatabase(XAPPLOADDIR "/" APL_SUBCLASS)) != NULL)
+	XrmMergeDatabases(rdb1, &database);
+
     XrmSetDatabase(display, database);
 #  endif
 
+/*
+ * Query resources for options that affect us
+ */
     for (entry = 0; entry < optList_size(); entry++) {
 	int             s;
 	char           *p;
@@ -770,10 +775,10 @@ extract_resources(Display *display, const char *name)
 	    *optList[entry].dp = p;
 
 	    if (optList_isBool(entry)) {
-		s = my_strcasecmp(p, "TRUE") == 0
-		    || my_strcasecmp(p, "YES") == 0
-		    || my_strcasecmp(p, "ON") == 0
-		    || my_strcasecmp(p, "1") == 0;
+		s = STRCASECMP(p, "TRUE") == 0
+		    || STRCASECMP(p, "YES") == 0
+		    || STRCASECMP(p, "ON") == 0
+		    || STRCASECMP(p, "1") == 0;
 		if (optList_isReverse(entry))
 		    s = !s;
 		if (s)
@@ -795,39 +800,30 @@ extract_resources(Display *display, const char *name)
     class_prefix[0] = XrmStringToName(APL_SUBCLASS);
     class_prefix[1] = XrmStringToName("Keysym");
     class_prefix[2] = NULLQUARK;
-    XrmEnumerateDatabase(XrmGetDatabase(display),
-			 name_prefix,
-			 class_prefix,
-			 XrmEnumOneLevel,
-			 define_key,
-			 NULL);
+    XrmEnumerateDatabase(XrmGetDatabase(display), name_prefix, class_prefix,
+			 XrmEnumOneLevel, define_key, NULL);
     name_prefix[0] = XrmStringToName(APL_CLASS);
     name_prefix[1] = XrmStringToName("keysym");
     class_prefix[0] = XrmStringToName(APL_CLASS);
     class_prefix[1] = XrmStringToName("Keysym");
-    XrmEnumerateDatabase(XrmGetDatabase(display),
-			 name_prefix,
-			 class_prefix,
-			 XrmEnumOneLevel,
-			 define_key,
-			 NULL);
+    XrmEnumerateDatabase(XrmGetDatabase(display), name_prefix, class_prefix,
+			 XrmEnumOneLevel, define_key, NULL);
 #   endif
 #  endif
 
 # else				/* USE_XGETDEFAULT */
 /* get resources the hard way, but save lots of memory */
-    const char     *const fname[] = { ".Xdefaults", ".Xresources" };
     FILE           *fd = NULL;
     char           *home;
 
     if ((home = getenv("HOME")) != NULL) {
-	int             i, len = strlen(home) + 2;
+	int             i, len = STRLEN(home) + 2;
 	char           *f = NULL;
 
-	for (i = 0; i < (sizeof(fname) / sizeof(fname[0])); i++) {
-	    f = REALLOC(f, (len + strlen(fname[i])) * sizeof(char));
+	for (i = 0; i < (sizeof(xnames) / sizeof(xnames[0])); i++) {
+	    f = REALLOC(f, (len + STRLEN(xnames[i])) * sizeof(char));
 
-	    sprintf(f, "%s/%s", home, fname[i]);
+	    sprintf(f, "%s/%s", home, xnames[i]);
 
 	    if ((fd = fopen(f, "r")) != NULL)
 		break;
@@ -859,7 +855,7 @@ extract_resources(Display *display, const char *name)
     get_xdefaults(fd, name);
     get_xdefaults(fd, APL_SUBCLASS);
 
-#  ifdef XAPPLOADDIR
+#  if defined(XAPPLOADDIR) && defined(USE_XAPPLOADDIR)
     {
 	FILE           *ad = fopen(XAPPLOADDIR "/" APL_SUBCLASS, "r");
 
@@ -868,7 +864,7 @@ extract_resources(Display *display, const char *name)
 	    fclose(ad);
 	}
     }
-#  endif				/* XAPPLOADDIR */
+#  endif			/* XAPPLOADDIR */
 
     get_xdefaults(fd, APL_CLASS);
     get_xdefaults(fd, "");	/* partial match */
@@ -882,15 +878,15 @@ extract_resources(Display *display, const char *name)
  * options and command-line long options
  */
 #ifdef MULTICHAR_SET
-    set_multichar_encoding(rs[Rs_multichar_encoding]);
+    set_multichar_encoding(rs.multichar_encoding);
 #endif
 #ifdef GREEK_SUPPORT
 /* this could be a function in grkelot.c */
 /* void set_greek_keyboard (const char * str); */
-    if (rs[Rs_greek_keyboard]) {
-	if (!strcmp(rs[Rs_greek_keyboard], "iso"))
+    if (rs.greek_keyboard) {
+	if (!STRCMP(rs.greek_keyboard, "iso"))
 	    greek_setmode(GREEK_ELOT928);	/* former -grk9 */
-	else if (!strcmp(rs[Rs_greek_keyboard], "ibm"))
+	else if (!STRCMP(rs.greek_keyboard, "ibm"))
 	    greek_setmode(GREEK_IBM437);	/* former -grk4 */
     }
 #endif				/* GREEK_SUPPORT */
@@ -899,14 +895,15 @@ extract_resources(Display *display, const char *name)
     {
 	KeySym          sym;
 
-	if (rs[Rs_bigfont_key]
-	    && ((sym = XStringToKeysym(rs[Rs_bigfont_key])) != 0))
+	if (rs.bigfont_key
+	    && ((sym = XStringToKeysym(rs.bigfont_key)) != 0))
 	    ks_bigfont = sym;
-	if (rs[Rs_smallfont_key]
-	    && ((sym = XStringToKeysym(rs[Rs_smallfont_key])) != 0))
+	if (rs.smallfont_key
+	    && ((sym = XStringToKeysym(rs.smallfont_key)) != 0))
 	    ks_smallfont = sym;
     }
 #endif
 }
+
 /*}}} */
 /*----------------------- end-of-file (C source) -----------------------*/
